//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.core.exporters;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.filechooser.FileFilter;

import net.ajaest.core.stylers.Styler;

//TODO: JavaDoc
/**
 * WARNING: exporters must be runtime-independent. They must be able to work
 * alone, what means, for example, that export windows and messages must be
 * stored in the exporter itself. That's because they are intended to use they
 * as plugins.
 */
public abstract class Exporter<E> implements Comparable<Exporter<?>>, Serializable {


	private static final long serialVersionUID = 6912511348890433716L;
	private File f;
	private String languaje;
	private Exception exportException;
	private JFrame dialogParent = null;
	private List<Styler<E>> stylers;
	private Integer selectedStyler;
	
	public abstract Boolean export(List<E> list);
	public abstract JDialog extraConfigJFrame();
	public abstract void invokeExtraConfigJFrame();
	public abstract String getName();
	protected abstract void initMessages(String lang);
	public abstract FileFilter getFileChooserFilters();

	public Exporter(String lang) {
		this.languaje = lang;
		stylers = new ArrayList<Styler<E>>();
	}

	public String getLanguaje(){
		return languaje;
	}

	public void setLanguaje(String lang) {
		this.languaje = lang.toLowerCase();
		initMessages(languaje);
		for (Styler<?> sty : stylers)
			sty.setLanguaje(languaje);
	}

	public void setExportPath(File f) {
		this.f = f;
	}

	public File getExportFile() {

		return f;
	}

	public void setExportFile(File f) {

		this.f = f;
	}

	public Exception getExportException() {
		return exportException;
	}
	protected void setExportException(Exception exportException) {
		this.exportException = exportException;
	}

	@Override
	public int compareTo(Exporter<?> exporter) {

		return getName().compareTo(exporter.getName());
	}
	public void setDialogParent(JFrame dialogParent) {
		this.dialogParent = dialogParent;
	}
	public JFrame getDialogParent() {
		return dialogParent;
	}
	public void setStylers(List<Styler<E>> stylers) {
		this.stylers = stylers;
	}
	public List<Styler<E>> getStylers() {
		return stylers;
	}

	public void setSelectedStyler(Integer selectedStyler) {
		this.selectedStyler = selectedStyler;
	}
	public Integer getSelectedStyler() {
		return selectedStyler;
	}
}
