//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.core.exporters;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JDialog;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;

import net.ajaest.core.stylers.KanjiPlainTextStyler;
import net.ajaest.data.kanji.KanjiTag;

//TODO: JavaDoc
public class KanjiPlainTextExporter extends Exporter<KanjiTag> {


	private static final long serialVersionUID = -8879756140144794244L;
	private Map<String, HashMap<String, String>> strings = null;
	private ExporterDialog configDialog = null;
	private KanjiPlainTextStyler textStyler;

	public KanjiPlainTextExporter(String lang) {
		super(lang);

		strings = new HashMap<String, HashMap<String, String>>();
		initMessages(getLanguaje());

		textStyler = new KanjiPlainTextStyler(lang);
		textStyler.setCompleteDescription(true);
		getStylers().add(textStyler);
	}

	@Override
	public Boolean export(List<KanjiTag> list) {

		Boolean success = false;

		String fileString = textStyler.stylerString(list);

		// System.out.println(fileString);

		if (getExportFile() != null && !getExportFile().getName().endsWith(".txt"))
			setExportFile(new File(getExportFile().getAbsolutePath() + ".txt"));

		try {
			// no buffered is needed because the string is written at once
			if (!getExportFile().exists())
				getExportFile().createNewFile();
			BufferedWriter fw = new BufferedWriter(new FileWriter(getExportFile()));
			fw.write(fileString);
			fw.flush();
			fw.close();
			success = true;
			super.setExportException(null);
		} catch (Exception e) {
			super.setExportException(e);
		}


		return success;
	}

	
	@Override
	public JDialog extraConfigJFrame() {
		
		if (getDialogParent() != null)
			configDialog = new ExporterDialog(getDialogParent(), true);

		return configDialog;
	}

	@Override
	public void invokeExtraConfigJFrame() {

		if (extraConfigJFrame() == null)
			throw new RuntimeException("Dialog parent is not defined");
		else
			SwingUtilities.invokeLater((ExporterDialog) extraConfigJFrame());

	}

	@Override
	public String getName() {
		return strings.get("localized").get("name");
	}

	@Override
	protected void initMessages(String lang) {
		strings = new HashMap<String, HashMap<String, String>>();

		//Spanish
		if(lang.equals("es")){

			HashMap<String, String> localized = new HashMap<String, String>();
			localized.put("name", "Archivo de texto");
			localized.put("fileDes", "Texto plano (.txt)");
			strings.put("localized", localized);

			HashMap<String, String> dialogMessages = new HashMap<String, String>();
			dialogMessages.put("acceptButton", "Aceptar");
			dialogMessages.put("cancelButton", "Cancelar");
			dialogMessages.put("extendedDescriptionJLabel", "Mostrar información extendida");
			dialogMessages.put("diaName", "Más...");
			strings.put("dialogMessages", dialogMessages);

		}else {
		
		// English otherwise

			HashMap<String, String> localized = new HashMap<String, String>();
			localized.put("name", "Text file");
			localized.put("fileDes", "Plain text (.txt)");
			strings.put("localized", localized);

			HashMap<String, String> dialogMessages = new HashMap<String, String>();
			dialogMessages.put("acceptButton", "Accept");
			dialogMessages.put("cancelButton", "Cancel");
			dialogMessages.put("extendedDescriptionJLabel", "Show extended info");
			dialogMessages.put("diaName", "More...");
			strings.put("dialogMessages", dialogMessages);
		}
	}

	@Override
	public FileFilter getFileChooserFilters() {
		
		return new FileFilter() {
			
			@Override
			public String getDescription() {
				return strings.get("localized").get("fileDes");
			}
			
			@Override
			public boolean accept(File f) {
				return f.getName().endsWith(".txt") || f.isDirectory();
			}
		};
	}

	public KanjiPlainTextStyler getTextStyler() {
		return textStyler;
	}

	public static void main(String... strings) throws IOException {

		KanjiPlainTextExporter kte = new KanjiPlainTextExporter("es");

		System.out.println(kte.getName());
		kte.setLanguaje("en");
		System.out.println(kte.getName());

		ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("exporters/textExporter.obj"));

		oos.writeObject(new KanjiPlainTextExporter("en"));
		oos.flush();
		oos.close();
		

		// Set<KanjiTag> asd = new KanjiDatabaseManager(new
		// File("dict/kanjidict.jdk"), new
		// File("dict/strokedata.txt")).executeQuery(new
		// KanjiQuery().grade().equalz(1));
		//		
		// KanjiPlainTextExporter kte = new KanjiPlainTextExporter("en");
		//				
		// kte.getTextStyler().setCompleteDescription(true);
		// System.out.println(kte.export(new ArrayList<KanjiTag>(asd)));
		// System.out.println(kte.getExportException());
		//		
		// JFrame dsa = new JFrame("test");
		// dsa.setSize(200, 200);
		// dsa.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		// dsa.setVisible(true);
		// kte.setDialogParent(dsa);
		// kte.runExtraConfigJFrame();

	}


	public class ExporterDialog extends javax.swing.JDialog implements Runnable {

		private static final long serialVersionUID = 321976699961363310L;
		
		
		public ExporterDialog(java.awt.Frame parent, boolean modal) {
			super(parent, modal);
			this.setResizable(false);
			this.setTitle(strings.get("dialogMessages").get("diaName"));
			this.setLocationRelativeTo(null);
			initComponents();
		}

		private void initComponents() {

			extendedDescriptionCheckBox = new javax.swing.JCheckBox();
			extendedDescriptionCheckBox.setSelected(textStyler.getCompleteDescription());
			extendedDescriptionJLabel = new javax.swing.JLabel();
			acceptButton = new javax.swing.JButton();
			cancelButton = new javax.swing.JButton();

			setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

			extendedDescriptionJLabel.setText(strings.get("dialogMessages").get("extendedDescriptionJLabel"));

			acceptButton.setText(strings.get("dialogMessages").get("acceptButton"));
			acceptButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					textStyler.setCompleteDescription(extendedDescriptionCheckBox.isSelected());
					dispose();
				}
			});

			cancelButton.setText(strings.get("dialogMessages").get("cancelButton"));
			cancelButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					setVisible(false);
					dispose();

				}
			});


			javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
	        getContentPane().setLayout(layout);
	        layout.setHorizontalGroup(
	            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
	                .addContainerGap()
	                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
	                    .addComponent(cancelButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 198, Short.MAX_VALUE)
	                    .addGroup(layout.createSequentialGroup()
	                        .addComponent(extendedDescriptionJLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 166, Short.MAX_VALUE)
	                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
	                        .addComponent(extendedDescriptionCheckBox))
	                    .addComponent(acceptButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 198, Short.MAX_VALUE))
	                .addContainerGap())
	        );
	        layout.setVerticalGroup(
	            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	            .addGroup(layout.createSequentialGroup()
	                .addGap(12, 12, 12)
	                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
	                    .addComponent(extendedDescriptionCheckBox)
	                    .addComponent(extendedDescriptionJLabel))
	                .addGap(18, 18, 18)
	                .addComponent(acceptButton)
	                .addGap(18, 18, 18)
	                .addComponent(cancelButton)
	                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
	        );
			pack();
		}
		private javax.swing.JButton acceptButton;
		private javax.swing.JButton cancelButton;
		private javax.swing.JCheckBox extendedDescriptionCheckBox;
		private javax.swing.JLabel extendedDescriptionJLabel;

		@Override
		public void run() {

			this.setVisible(true);
		}

	}

}
