//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.core.main;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;

import javax.swing.SwingUtilities;

import net.ajaest.core.exporters.Exporter;
import net.ajaest.core.main.ExceptionHandler.KNOWN_EXCEPTIONS;
import net.ajaest.data.auxi.SystemEnums.ISO639ー1;
import net.ajaest.data.dict.KanjiDatabaseInfo;
import net.ajaest.data.dict.KanjiDict;
import net.ajaest.data.dict.query.KanjiExpression;
import net.ajaest.data.dict.query.QueryAdaptor;
import net.ajaest.data.kanji.KanjiTag;
import net.ajaest.data.kanji.Radical;
import net.ajaest.data.kanji.SimilarKanjiStrokeDemuxer;

//TODO: JavaDoc 
public class JavaDiKt implements Runnable {

	// Program engine
	private JDKOptions jdkOpt;
	private JDKGUIEngine jdkGui;
	private KanjiDict kd;
	private List<Exporter<KanjiTag>> exporters;
	private SimilarKanjiStrokeDemuxer sksdmx;

	public static final String VERSION = "1.0 beta";
	public static final String PROJECT = "Mirai";

	// Program fields

	public JavaDiKt(JDKOptions jdkOpt) {

		this.jdkOpt = jdkOpt;

		try {
			kd = new KanjiDict(jdkOpt.getKanjiFile(), jdkOpt.getStrokeFile());

		} catch (FileNotFoundException e) {
			ExceptionHandler.handleException(e, KNOWN_EXCEPTIONS.INVALID_KANJI_OR_STROKE_FILE_PATH);
		} catch (IOException e) {

		}

		jdkGui = new JDKGUIEngine(this);

	}

	@Override
	public void run() {
		SwingUtilities.invokeLater(jdkGui);
	}

	public List<KanjiTag> executeQuery(KanjiExpression currentKE) {
		List<KanjiTag> results = kd.executeQuery(adaptQuery(currentKE));

		return new ArrayList<KanjiTag>(results);
	}

	@SuppressWarnings("static-access")
	private KanjiExpression adaptQuery(KanjiExpression ke) {

		QueryAdaptor qa = QueryAdaptor.demuxGraphs(sksdmx, 5).meaningsInLanguaje(ISO639ー1.valueOf(jdkOpt.getLanguage().toUpperCase()));
		
		if (!jdkOpt.getExtendedIKanjiList()) {
			qa = qa.limitToJôyô();
		}

		return qa.adapt(ke);
	}

	public JDKOptions getOptions() {
		return jdkOpt;
	}

	public void saveOptions(String lang, String kanjiDictpath, String strokeDictPath, Boolean extInfo, Boolean extKanji, Boolean romaji) {

		jdkOpt.setLanguage(lang);
		jdkOpt.setKanjiFile(new File(kanjiDictpath));
		jdkOpt.setStrokeFile(new File(strokeDictPath));
		jdkOpt.setExtendedInfo(extInfo);
		jdkOpt.setFindInExtended(extKanji);
		jdkOpt.setRomanized(romaji);
		jdkOpt.saveCurrentConfig();
	}

	public void setExporters(List<Exporter<KanjiTag>> exporters) {

		for (Exporter<KanjiTag> exp : exporters) {
			exp.setLanguaje(jdkOpt.getLanguage());
		}

		this.exporters = exporters;
	}

	public List<Exporter<KanjiTag>> getExporters() {
		return exporters;
	}

	public static void main(String... args) {

		Launcher.main();
	}

	public Radical getRadicalInfo(Integer rad) {

		return kd.getRadicalMainInfo(rad);
	}

	public SortedSet<Radical> getRadicalInfos(Integer rad) {

		return kd.getRadicalInfos(rad);
	}

	public JDKGUIEngine getJDKGuiEngine() {
		return jdkGui;
	}
	public KanjiDatabaseInfo getKanjiDatabaseInfo() {
		return kd.getDatabaseInfo();
	}

	public SimilarKanjiStrokeDemuxer getSimilarGraphDemuxer() {
		return sksdmx;
	}

	public void setSimilarStrokeDemuxer(SimilarKanjiStrokeDemuxer sksdmx) {
		this.sksdmx = sksdmx;
	}

}
