//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.core.main;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import net.ajaest.core.exception.NoSuchMessage;
import net.ajaest.core.main.ExceptionHandler.KNOWN_EXCEPTIONS;

//TODO: translate into english
/**
 * 
 * Clase que comprende todos los métodos de entrada-salida y de tratamiento de
 * cadenas localizadas. Para usar los métodos de recuperación de cadenas la
 * clase primero ha de ser inicializada mediante el método estático {@code
 * initMessages()}. No está permitida la instanciación.
 * 
 */
public class Messages {

	private static Map<String, String> messages;
	private static Map<String, Map<String, String>> listMessages;
	
	private static Map<String, List<String>> keyMap;

	private static boolean initialized = false;
	
	public final static String[] SUPPORTED = {"EN","ES"};
	public final static String DEFAULT_LENGUAGE = SUPPORTED[0]; //EN
	public static String SYSTEM_LENGUAGE = System.getProperty("user.language").toUpperCase();
	public static String LENGUAGE;

	// Generic
	
	public final static String GEN_DIR_FILES_READ_ERROR_C = "DIR_FILES_READ_ERROR_C";
	public final static String GEN_GENERIC_READ_ERROR_C_colon = "GENERIC_READ_ERROR_C_colon";
	public final static String GEN_EXIT_SUCESS_C = "EXIT_SUCESS_C";
	public final static String GEN_EXIT_SUCCES_TIME_C = "EXIT_SUCCES_TIME_C";
	public final static String GEN_INDEXING = "INDEXING";
	public final static String GEN_LENGUAJE_DOES_NOT_EXISTS_C = "LENGUAJE_DOES_NOT_EXISTS";
	public final static String GEN_LOCAL_LABEL = "LOCAL_LABEL";
	public final static String GEN_NAME_ERROR = "NAME_ERROR";
	public final static String GEN_NAME_OR_PATH_ERROR = "NAME_OR_PATH_ERROR";
	public final static String GEN_NULL_POINTER = "NULL_POINTER";
	public final static String GEN_NOT_ISO_639_1 = "NO_ISO-639-1";
	public final static String GEN_NOT_MD5_HASH = "NOT_A_MD5_HASH";
	public final static String GEN_PATH_DOES_NOT_EXISTS = "PATH_DOES_NOT_EXISTS";
	public final static String GEN_PATH_ERROR = "PATH_ERROR";
	public final static String GEN_READ_ERROR_C = "READ_ERROR_C";
	public final static String GEN_RECOVERING_FILE_SYSTEM = "RECOVERING_FILE_SYSTEM";
	public final static String GEN_UNKNOWN_ERROR = "UNKNOWN_ERROR";
	public final static String GEN_WRITING = "WRITING";
	
	// LOCALIZED LANGUAGE NAMES

	public final static String LANGUAJE_EN = "LANGUAJE_EN";
	public final static String LANGUAJE_ES = "LANGUAJE_ES";

	// FILE DESCRIPTIONS

	public final static String FILE_DESCRIPTION_JDK_KANJI_FILE = "FILE_DESCRIPTION_JDK_KANJI_FILE";
	public final static String FILE_DESCRIPTION_TXT_STROKE_FILE = "FILE_DESCRIPTION_TXT_STROKE_FILE";

	// KANJI PROPERTIES

	public final static String KANJI_CLASSIC_NELSON = "KANJI_CLASSIC_NELSON";
	public final static String KANJI_CLASSIC_RADICAL = "KANJI_CLASSIC_RADICAL";
	public final static String KANJI_DE_ROO = "KANJI_DE_ROO";
	public final static String KANJI_DIC_NAME = "KANJI_DIC_NAME";
	public final static String KANJI_DIC_INDEX = "KANJI_DIC_INDEX";
	public final static String KANJI_FOUR_CORNER = "KANJI_FOUR_CORNER";
	public final static String KANJI_FREQUENCY = "KANJI_FREQUENCY";
	public final static String KANJI_GRADE = "KANJI_GRADE";
	public final static String KANJI_GRAPH = "KANJI_GRAPH";
	public final static String KANJI_JIS_CHARSET = "KANJI_JIS_CHARSET";
	public final static String KANJI_JIS_CODE = "KANJI_JIS_CODE";
	public final static String KANJI_JLPT_LEVEL = "KANJI_JLPT_LEVEL";
	public final static String KANJI_MEANING = "KANJI_MEANING";
	public final static String KANJI_READING_TYPE = "KANJI_READING_TYPE";
	public final static String KANJI_READING = "KANJI_READING";
	public final static String KANJI_SKIP = "KANJI_SKIP";
	public final static String KANJI_SPAHN_HADAMITZKY = "KANJI_SPAHN_HADAMITZKY";
	public final static String KANJI_STROKE_COUNT = "KANJI_STROKE_COUNT";
	public final static String KANJI_UNICODE_VALUE = "KANJI_UNICODE_VALUE";
	public final static String KANJI_VARIANT_INDEX = "KANJI_VARIANT_INDEX";
	public final static String KANJI_VARIANT_TYPE = "KANJI_VARIANT_TYPE";
	public final static String KANJI_FIRST_MEANING = "KANJI_FIRST_MEANING";
	public final static String KANJI_FIRST_READING = "KANJI_FIRST_READING";
	public final static String KANJI_DIC_REFERENCE = "KANJI_DIC_REFERENCE";
	
	//RADICAL TYPES

	public final static String RADICAL_TYPE_original = "RADICAL_TYPE_original";
	public final static String RADICAL_TYPE_top = "RADICAL_TYPE_top";
	public final static String RADICAL_TYPE_bottom = "RADICAL_TYPE_bottom";
	public final static String RADICAL_TYPE_left = "RADICAL_TYPE_left";
	public final static String RADICAL_TYPE_right = "RADICAL_TYPE_right";
	public final static String RADICAL_TYPE_enclosure = "RADICAL_TYPE_enclosure";
	public final static String RADICAL_TYPE_variable = "RADICAL_TYPE_variable";

	// ALLOWED CASES

	public final static String CASE_EQUALS = "CASE_EQUALS";
	public final static String CASE_NOT_EQUALS = "CASE_NOT_EQUALS";
	public final static String CASE_GREATHER = "CASE_GREATHER";
	public final static String CASE_LESS = "CASE_LESS";
	public final static String CASE_EQUALS_OR_GREATHER = "CASE_EQUALS_OR_GREATHER";
	public final static String CASE_EQUALS_OR_LESS = "CASE_EQUALS_OR_LESS";
	public final static String CASE_NULL = "CASE_NULL";

	// JIS CHARSET

	public final static String JIS_208 = "JIS_208";
	public final static String JIS_212 = "JIS_212";
	public final static String JIS_213 = "JIS_213";

	// PROGRAM LISTS
	
	public final static String LIST_CRITERIA_FIELD_BASIC = "LIST_CRITERIA_FIELD_BASIC";
	public final static String LIST_CRITERIA_FIELD_EXTENDED = "LIST_CRITERIA_FIELD_EXTENDED";
	public final static String LIST_CRITERIA_CASES_STRING = "LIST_CRITERIA_CASES_STRING";
	public final static String LIST_CRITERIA_CASES_INT = "LIST_CRITERIA_CASES_INT";
	public final static String LIST_CRITERIA_VALUES_JIS = "LIST_CRITERIA_VALUES_JIS";
	public final static String LIST_CRITERIA_VALUES_DIC = "LIST_CRITERIA_VALUES_DIC";
	public final static String LIST_CRITERIA_VALUES_READING_BASIC = "LIST_CRITERIA_VALUES_READING_BASIC";
	public final static String LIST_CRITERIA_VALUES_READING_EXTENDED = "LIST_CRITERIA_VALUES_READING_EXTENDED";
	public final static String LIST_LANGUAGES_SUPPORTED = "LIST_LANGUAGES_SUPPORTED";
	public final static String LIST_ORDENABLE_FIELDS = "LIST_ORDENABLE_FIELDS";
	public final static String LIST_ORDENABLE_METHODS_STRING = "LIST_ORDENABLE_METHODS_STRING";
	public final static String LIST_ORDENABLE_METHODS_INTEGER = "LIST_ORDENABLE_METHODS_INTEGER";
	
	// DICTIONARIES
	
	public final static String DIC_nelson_c = "DIC_nelson_c";
	public final static String DIC_nelson_n = "DIC_nelson_n";
	public final static String DIC_halpern_njecd = "DIC_halpern_njecd";
	public final static String DIC_halpern_kkld = "DIC_halpern_kkld";
	public final static String DIC_heisig = "DIC_heisig";
	public final static String DIC_gakken = "DIC_gakken";
	public final static String DIC_kanji_basic_book = "DIC_kanji_basic_book";
	public final static String DIC_oneill_names = "DIC_oneill_names";
	public final static String DIC_oneill_kk = "DIC_oneill_kk";
	public final static String DIC_moro = "DIC_moro";
	public final static String DIC_henshall = "DIC_henshall";
	public final static String DIC_sh_kk = "DIC_sh_kk";
	public final static String DIC_sakade = "DIC_sakade";
	public final static String DIC_jf_cards = "DIC_jf_cards";
	public final static String DIC_henshall3 = "DIC_henshall3";
	public final static String DIC_tutt_cards = "DIC_tutt_cards";
	public final static String DIC_crowley = "DIC_crowley";
	public final static String DIC_kanji_in_context = "DIC_kanji_in_context";
	public final static String DIC_busy_people = "DIC_busy_people";
	public final static String DIC_kodansha_compact = "DIC_kodansha_compact";
	public final static String DIC_maniette = "DIC_maniette";
	
	// READINGS
	
	public final static String READ_JA_ON = "READ_JA_ON";
	public final static String READ_JA_KUN = "READ_JA_KUN";
	public final static String READ_NANORI = "READ_NANORI";
	public final static String READ_PINYIN = "READ_PINYIN";
	public final static String READ_KOREAN_H = "READ_KOREAN_H";
	public final static String READ_KOREAN_R = "READ_KOREAN_R";
	
	// ORDENABLE PROPERTIES

	public final static String ORDER_UNICODE_VALUE = "ORDER_UNICODE_VALUE";
	public final static String ORDER_JIS_CHARSET = "ORDER_JIS_CHARSET";
	public final static String ORDER_JIS_CODE = "ORDER_JIS_CODE";
	public final static String ORDER_CLASSIC_NELSON = "ORDER_CLASSIC_NELSON";
	public final static String ORDER_CLASSIC_RADICAL = "ORDER_CLASSIC_RADICAL";
	public final static String ORDER_GRADE = "ORDER_GRADE";
	public final static String ORDER_STROKE_COUNT = "ORDER_STROKE_COUNT";
	public final static String ORDER_FREQUENCY = "ORDER_FREQUENCY";
	public final static String ORDER_SKIP = "ORDER_SKIP";
	public final static String ORDER_SPAHN_HADAMITZKY = "ORDER_SPAHN_HADAMITZKY";
	public final static String ORDER_FOUR_CORNER = "ORDER_FOUR_CORNER";
	public final static String ORDER_DE_ROO = "ORDER_DE_ROO";
	public final static String ORDER_GRAPH = "ORDER_GRAPH";
	public final static String ORDER_DIC_INDEX = "ORDER_DIC_INDEX";
	public final static String ORDER_MEANING = "ORDER_MEANING";
	public final static String ORDER_READING = "ORDER_READING";

	// ORDENABLE METHODS

	public final static String ORDERMETHODSSTR_ALPHABETICALLY = "ORDERMETHODSSTR_ALPHABETICALLY";
	public final static String ORDERMETHODSSTR_ALPHABETICALLY_INVERSE = "ORDERMETHODSSTR_ALPHABETICALLY_INVERSE";
	public final static String ORDERMETHODSINT_FROM_LEAST_TO_GREATEST = "ORDERMETHODSINT_FROM_LEAST_TO_GREATEST";
	public final static String ORDERMETHODSINT_FROM_GREATEST_TO_LEAST = "ORDERMETHODSINT_FROM_GREATEST_TO_LEAST";
	public final static String ORDERMETHODSINT_RAMDOM = "ORDERMETHODSINT_RAMDOM";
	public final static String ORDERMETHODSSTR_RAMDOM = "ORDERMETHODSSTR_RAMDOM";

	// WINMAIN
	
		// Window name
	
	public final static String WINMAIN_name = "WINMAIN_name";

		// Option tab names
	public final static String WINMAIN_TAB_CONTROL_About = "WINMAIN_TAB_CONTROL_About";
	public final static String WINMAIN_TAB_CONTROL_Criteria = "WINMAIN_TAB_CONTROL_Criteria";
	public final static String WINMAIN_TAB_CONTROL_Drawing = "WINMAIN_TAB_CONTROL_Drawing";
	public final static String WINMAIN_TAB_CONTROL_Export = "WINMAIN_TAB_CONTROL_Export";
	public final static String WINMAIN_TAB_CONTROL_Config = "WINMAIN_TAB_CONTROL_Config";

		// Border texts
	public final static String WINMAIN_CRITERIA_BorderTextQueryData = "WINMAIN_CRITERIA_BorderTextQueryData";
	public final static String WINMAIN_CRITERIA_BorderTextQueryMaker = "WINMAIN_CRITERIA_BorderTextQueryMaker";
	public final static String WINMAIN_CRITERIA_BorderTextQueryControls = "WINMAIN_CRITERIA_BorderTextQueryControls";
	public final static String WINMAIN_RESULTS_BorderTextResults = "WINMAIN_RESULTS_BorderTextResults";
	public final static String WINMAIN_RESULTS_EXPORT_BorderTextControls = "WINMAIN_RESULTS_EXPORT_BorderTextControls";
	public final static String WINMAIN_STROKES_BorderTextControls = "WINMAIN_STROKES_BorderTextControls";
	public final static String WINMAIN_STROKES_BorderTextAndOr = "WINMAIN_STROKES_BorderTextAndOr";
	public final static String WINMAIN_EXPORT_BorderTextExportAs = "WINMAIN_EXPORT_BorderTextExportAs";
	public final static String WINMAIN_EXPORT_BorderTextExportSorting = "WINMAIN_EXPORT_BorderTextExportSorting";
	public final static String WINMAIN_EXPORT_BorderTextExportExtra = "WINMAIN_EXPORT_BorderTextExportExtra";
	
		// Query table column names
	public final static String WINMAIN_QUERY_DATA_columnAndOr = "WINMAIN_QUERY_DATA_columnAndOr";
	public final static String WINMAIN_QUERY_DATA_columnField = "WINMAIN_QUERY_DATA_columnField";
	public final static String WINMAIN_QUERY_DATA_columnThatFulfilsThat = "WINMAIN_QUERY_DATA_columnThatFulfilsThat";
	public final static String WINMAIN_QUERY_DATA_columnValue = "WINMAIN_QUERY_DATA_columnValue";
	public final static String WINMAIN_EXPORT_SORTING_columnLevel = "WINMAIN_EXPORT_SORTING_columnLevel";
	public final static String WINMAIN_EXPORT_SORTING_columnOrderBy = "WINMAIN_EXPORT_SORTING_columnOrderBy";
	public final static String WINMAIN_EXPORT_SORTING_columnField = "WINMAIN_EXPORT_SORTING_columnField";
	
		//Query table cell string
	
	public final static String WINMAIN_QUERY_DATA_cellAndString = "WINMAIN_QUERY_DATA_cellAndString";
	public final static String WINMAIN_QUERY_DATA_cellOrString = "WINMAIN_QUERY_DATA_cellOrString";
	public final static String WINMAIN_QUERY_DATA_cellFirstString = "WINMAIN_QUERY_DATA_cellFirstString";
	
		// Text Label
	public final static String WINMAIN_CRITERIA_LabelFulfilsComboBox = "WINMAIN_CRITERIA_LabelFulfilsComboBox";
	public final static String WINMAIN_CRITERIA_LabelValueComboBox = "WINMAIN_CRITERIA_LabelValueComboBox";
	public final static String WINMAIN_EXPORT_LabelFormatComboBox = "WINMAIN_EXPORT_LabelFormatComboBox";
	public final static String WINMAIN_EXPORT_LabelStyleComboBox = "WINMAIN_EXPORT_LabelStyleComboBox";
	public final static String WINMAIN_EXPORT_LabelDirButton = "WINMAIN_EXPORT_LabelDirButton";
	public final static String WINMAIN_CRITERIA_LabelFieldComboBox = "WINMAIN_CRITERIA_LabelFieldComboBox";
	public final static String WINMAIN_OPTIONS_LabelLanguajeComboBox = "WINMAIN_OPTIONS_LabelLanguajeComboBox";
	public final static String WINMAIN_OPTIONS_LabelKanjiFileTextField = "WINMAIN_OPTIONS_LabelKanjiFileTextField";
	public final static String WINMAIN_OPTIONS_LabelStrokeFileTextField = "WINMAIN_OPTIONS_LabelStrokeFileTextField";
	public final static String WINMAIN_OPTIONS_LabelExtendedInfoCheckBox = "WINMAIN_OPTIONS_LabelExtendedInfoCheckBox";
	public final static String WINMAIN_OPTIONS_LabelExtendedKanjiListCheckBox = "WINMAIN_OPTIONS_LabelExtendedKanjiListCheckBox";
	public final static String WINMAIN_OPTIONS_LabelRomajiCheckBox = "WINMAIN_OPTIONS_LabelRomajiCheckBox";
	
		// Button text

	public final static String WINMAIN_CRITERIA_ButtonQueryMakerAndText = "WINMAIN_CRITERIA_ButtonQueryMakerAndText";
	public final static String WINMAIN_CRITERIA_ButtonQueryMakerAddText = "WINMAIN_CRITERIA_ButtonQueryMakerAddText";
	public final static String WINMAIN_CRITERIA_ButtonQueryMakerChangeText = "WINMAIN_CRITERIA_ButtonQueryMakerChangeText";
	public final static String WINMAIN_CRITERIA_ButtonQueryMakerOrText = "WINMAIN_CRITERIA_ButtonQueryMakerOrText";
	public final static String WINMAIN_CRITERIA_ButtonQueryControlResetText = "WINMAIN_CRITERIA_ButtonQueryControlResetText";
	public final static String WINMAIN_CRITERIA_ButtonQueryControlRemoveText = "WINMAIN_CRITERIA_ButtonQueryControlRemoveText";
	public final static String WINMAIN_CRITERIA_ButtonQueryControlModifyText = "WINMAIN_CRITERIA_ButtonQueryControlModifyText";
	public final static String WINMAIN_CRITERIA_ButtonQueryControlExecuteText = "WINMAIN_CRITERIA_ButtonQueryControlExecuteText";
	public final static String WINMAIN_STROKES_ButtonDrawPanelReset = "WINMAIN_STROKES_ButtonDrawPanelReset";
	public final static String WINMAIN_STROKES_ButtonDrawPanelRemoveLastStroke = "WINMAIN_STROKES_ButtonDrawPanelRemoveLastStroke";
	public final static String WINMAIN_STROKES_ButtonDrawPanelRemoveLastLine = "WINMAIN_STROKES_ButtonDrawPanelRemoveLastLine";
	public final static String WINMAIN_RESULTS_ButtonExportAll = "WINMAIN_RESULTS_ButtonExportAll";
	public final static String WINMAIN_RESULTS_ButtonExportInvert = "WINMAIN_RESULTS_ButtonExportInvert";
	public final static String WINMAIN_RESULTS_ButtonExportReset = "WINMAIN_RESULTS_ButtonExportReset";
	public final static String WINMAIN_EXPORT_ButtonExportExtraOptions = "WINMAIN_EXPORT_ButtonExportExtraOptions";
	public final static String WINMAIN_EXPORT_ButtonSortingAdd = "WINMAIN_EXPORT_ButtonSortingAdd";
	public final static String WINMAIN_EXPORT_ButtonSortingReset = "WINMAIN_EXPORT_ButtonSortingReset";
	public final static String WINMAIN_EXPORT_ButtonSortingRemove = "WINMAIN_EXPORT_ButtonSortingRemove";
	public final static String WINMAIN_EXPORT_ButtonSortingModify = "WINMAIN_EXPORT_ButtonSortingModify";
	public final static String WINMAIN_EXPORT_ButtonExtraRearrange = "WINMAIN_EXPORT_ButtonExtraRearrange";
	public final static String WINMAIN_EXPORT_ButtonExtraOptions = "WINMAIN_EXPORT_ButtonExtraOptions";
	public final static String WINMAIN_EXPORT_ButtonExport = "WINMAIN_EXPORT_ButtonExport";
	public final static String WINMAIN_OPTIONS_ButtonSave = "WINMAIN_OPTIONS_ButtonSave";
	public final static String WINMAIN_OPTIONS_ButtonAbout = "WINMAIN_OPTIONS_ButtonAbout";

	// Export sorter adder
		//Window name
	public final static String WINEXPORTRESORTERADDER_name = "WINEXPORTRESORTERADDER_name";
		//Buttons
	public final static String WINEXPORTRESORTERADDER_ButtonAdd = "WINEXPORTRESORTERADDER_ButtonAdd";
	public final static String WINEXPORTRESORTERADDER_ButtonCancel = "WINEXPORTRESORTERADDER_ButtonCancel";
	// Labels
	public final static String WINEXPORTRESORTERADDER_LabelLevel = "WINEXPORTRESORTERADDER_LabelLevel";
	public final static String WINEXPORTRESORTERADDER_LabelOrderBy = "WINEXPORTRESORTERADDER_LabelOrderBy";
	public final static String WINEXPORTRESORTERADDER_LabelField = "WINEXPORTRESORTERADDER_LabelField";
	
	// Export rearranger Dialog
		// Window name
	public final static String WINEXPORTREARRANGER_name = "WINEXPORTREARRANGER_name";
		// Buttons
	public final static String WINEXPORTREARRANGER_acceptButton = "WINEXPORTREARRANGER_acceptButton";
	public final static String WINEXPORTREARRANGER_cancelButton = "WINEXPORTREARRANGER_cancelButton";
		// table column names
	public final static String WINEXPORTREARRANGER_kanjiLiteral = "WINEXPORTREARRANGER_kanjiLiteral";
	
	// Kanji info window
		// Panel titles
	
	public final static String WINKANJIINFO_PanelTitle_Radical = "WINKANJIINFO_PanelTitle_Radical";
	public final static String WINKANJIINFO_PanelTitle_StrokeCount = "WINKANJIINFO_PanelTitle_StrokeCount";
	public final static String WINKANJIINFO_PanelTitle_Grade = "WINKANJIINFO_PanelTitle_Grade";
	public final static String WINKANJIINFO_PanelTitle_JLPTLevel = "WINKANJIINFO_PanelTitle_JLPTLevel";
	public final static String WINKANJIINFO_PanelTitle_Frequency = "WINKANJIINFO_PanelTitle_Frequency";
	public final static String WINKANJIINFO_PanelTitle_NelsonRadical = "WINKANJIINFO_PanelTitle_NelsonRadical";
	public final static String WINKANJIINFO_PanelTitle_Unicode = "WINKANJIINFO_PanelTitle_Unicode";
	public final static String WINKANJIINFO_PanelTitle_JIS = "WINKANJIINFO_PanelTitle_JIS";
	public final static String WINKANJIINFO_PanelTitle_DicReferences = "WINKANJIINFO_PanelTitle_DicReferences";
	public final static String WINKANJIINFO_PanelTitle_Graph = "WINKANJIINFO_PanelTitle_Graph";
	public final static String WINKANJIINFO_PanelTitle_Meanings = "WINKANJIINFO_PanelTitle_Meanings";
	public final static String WINKANJIINFO_PanelTitle_Readings = "WINKANJIINFO_PanelTitle_Readings";
	public final static String WINKANJIINFO_PanelTitle_Variants = "WINKANJIINFO_PanelTitle_Variants";
	public final static String WINKANJIINFO_PanelTitle_QueryCodes = "WINKANJIINFO_PanelTitle_QueryCodes";
	public final static String WINKANJIINFO_PanelTitle_QueryCodes_SKIP = "WINKANJIINFO_PanelTitle_QueryCodes_SKIP";
	public final static String WINKANJIINFO_PanelTitle_QueryCodes_FourCorner = "WINKANJIINFO_PanelTitle_QueryCodes_FourCorner";
	public final static String WINKANJIINFO_PanelTitle_QueryCodes_SpahnHadamitzky = "WINKANJIINFO_PanelTitle_QueryCodes_SpahnHadamitzky";
	public final static String WINKANJIINFO_PanelTitle_QueryCodes_DeRoo = "WINKANJIINFO_PanelTitle_QueryCodes_DeRoo";

		// Radical Info panel
	public final static String PANELRADICALINFO_radicalNumber = "PANELRADICALINFO_radicalNumber";
	public final static String PANELRADICALINFO_radicalType = "PANELRADICALINFO_radicalType";
	public final static String PANELRADICALINFO_radicalNames = "PANELRADICALINFO_radicalNames";
	public final static String PANELRADICALINFO_radicalUnicode = "PANELRADICALINFO_radicalUnicode";
	public final static String PANELRADICALINFO_radicalUnicodeName = "PANELRADICALINFO_radicalUnicodeName";
	public final static String PANELRADICALINFO_radicalKangxi = "PANELRADICALINFO_radicalKangxi";

	// INFO DIALOG
		// title
	public final static String INFODIALOG_title_error = "INFODIALOG_title_error";
	public final static String INFODIALOG_title_info = "INFODIALOG_title_info";
		// Messages
	public final static String INFODIALOG_buttonAccept = "INFODIALOG_buttonAccept";
	public final static String INFODIALOG_errorMessage_exportFail = "INFODIALOG_errorMessage_exportFail";
	public final static String INFODIALOG_errorMessage_castException = "INFODIALOG_errorMessage_castException";
	public final static String INFODIALOG_infoMessage_exportSuccess = "INFODIALOG_infoMessage_exportSuccess";
	public final static String INFODIALOG_infoMessage_functionalityNotAvaible = "INFODIALOG_infoMessage_functionalityNotAvaible";

	// ABOUT WINDOW
		// Title
	public final static String ABOUTWIN_title = "ABOUTWIN_title";
	public final static String ABOUTWIN_javadicText = "ABOUTWIN_javadicText";
	public final static String ABOUTWIN_borderTextLicense = "ABOUTWIN_borderTextLicense";
	public final static String ABOUTWIN_licenseText = "ABOUTWIN_licenseText";

	//-------------------
	private Messages(){}//impide inicializar objetos de tipo Messages

	/**
	 * 
	 * <br>
	 * <b> initMessages</b>
	 * 
	 * <br>
	 * <br>{@code public final static void initMessages()}
	 * 
	 * <br>
	 * <br>
	 * Método que inicializa la clase Messages en el idioma del sistema,
	 * permitiendo obtener desde get las cadenas almacenadas. Si el idioma del
	 * sistema no está soportado, inicia el idioma por defecto señalado en
	 * DEFAULT_LENGUAGE.<br>
	 * <br>
	 */
	public final static void initMessages() { 
		
		//No se implementa ningún método para evitar llamadas recursivas infinitas
		// en caso de que el idioma por defecto no tenga el formato correcto, o
		// por si no existe el idioma por defecto, eso deberá estar siempre
		// solucionado en tiempo de compilación.

		initMessages(SYSTEM_LENGUAGE);	
	}

	/**
	 * 
	 * <br>
	 * <b> initMessages</b>
	 * 
	 * <br>
	 * <br>{@code public final static void initMessages()}
	 * 
	 * <br>
	 * <br>
	 * Método que inicializa la clase Messages en el idioma elegido de la lista
	 * ISO-639-1 permitiendo obtener desde get las cadenas almacenadas.<br>
	 * <br>
	 * 
	 * @param languaje
	 *            - Cadena con el código del lenguaje en formato ISO-639-1.
	 * @throws IllegalArgumentException
	 *             - Si la cadena es nula.
	 */
	public final static void initMessages(String languaje) { 
		// Nombre de error termminado en:
		// "~colon": Prosigue un mensaje después de la descripción del error.
		// "~_C": El mensaje está formateado según el estándar de printf de C
		
		if(languaje == null)
			throw new IllegalArgumentException(get("NULL_POINTER")); //no inicializa el idioma por defecto,un fallo de puntero nulo debe ser resuelto en tiempo de compilación
		
		if( languaje.length()!=2 ||	
			!Character.isLetter(languaje.charAt(0)) ||
			!Character.isLetter(languaje.charAt(1))) {
			
			initMessages(); //inicializa el idioma por defecto, no es realmente necesario

			localePrintln(get("NO_ISO-639-1"));
		
		//BLOQUES DE IDIOMAS
			
		} else if (languaje.toUpperCase().equals("EN")) {
			
			LENGUAGE = "EN";
			
			if (initialized = true) {
				messages = new HashMap<String, String>();
				listMessages = new HashMap<String, Map<String, String>>();
			}
			// Generic
			
			messages.put(GEN_DIR_FILES_READ_ERROR_C, "**Could not access to files in folder \"%s\", maybe the file does not exist, it is locked or you do not have enough read privileges.");
			messages.put(GEN_GENERIC_READ_ERROR_C_colon, "**The file or folder \"%s\" could not be read: ");
			messages.put(GEN_EXIT_SUCESS_C, "The synchronization file was sucessfuly built in \"%s\".");
			messages.put(GEN_EXIT_SUCCES_TIME_C, "Runtime:[%s:%s:%s]");
			messages.put(GEN_INDEXING, "[Indexing...]\n");
			messages.put(GEN_LENGUAJE_DOES_NOT_EXISTS_C, "**The requested lenguage \"%s\" does not exist or is not defined, initializing default lenguage settings.");
			messages.put(GEN_LOCAL_LABEL, "After the program name, type the directory path wich will be used to build the synchronization file and following the synchronization file name.");
			messages.put(GEN_NAME_ERROR, "**The typed filename is not valid.");
			messages.put(GEN_NAME_OR_PATH_ERROR, "**The typed file name or directory path is not valid.");
			messages.put(GEN_NOT_ISO_639_1, "**The lenguage string has not ISO-639-1 format, initializing default lenguaje settings.");
			messages.put(GEN_NOT_MD5_HASH, "Not a MD5 hash.");
			messages.put(GEN_NULL_POINTER, "**Null pointer error.");
			messages.put(GEN_PATH_DOES_NOT_EXISTS, "**The typed diectory path do not exists.");
			messages.put(GEN_PATH_ERROR, "**The typed directory path name is not valid.");
			messages.put(GEN_READ_ERROR_C, "**The file or folder %s can not be read, maybe the file does not exists or you do not have enough read privileges.");
			messages.put(GEN_RECOVERING_FILE_SYSTEM, "\n[Recovering file system...]\n");
			messages.put(GEN_UNKNOWN_ERROR, "**Unknown error.");
			messages.put(GEN_WRITING, "[Writing synchronization file...]\n");

			// FILE DESCRIPTIONS

			messages.put(FILE_DESCRIPTION_JDK_KANJI_FILE, "JavaDiKt .jdk kanji file");
			messages.put(FILE_DESCRIPTION_TXT_STROKE_FILE, "JavaDiKt .txt stroke file");

			// PROGRAM LISTS

			Map<String, String> criteriaFieldsBasic = new HashMap<String, String>();
			criteriaFieldsBasic.put(KANJI_CLASSIC_RADICAL,"Radical");
			criteriaFieldsBasic.put(KANJI_FREQUENCY,"Frequency");
			criteriaFieldsBasic.put(KANJI_DIC_NAME,"Dictionary name");
			criteriaFieldsBasic.put(KANJI_DIC_INDEX,"Dictionary index");
			criteriaFieldsBasic.put(KANJI_GRADE,"Grade");
			criteriaFieldsBasic.put(KANJI_JLPT_LEVEL,"JLPT level");
			criteriaFieldsBasic.put(KANJI_MEANING,"Meaning");
			criteriaFieldsBasic.put(KANJI_READING_TYPE, "Reading type");
			criteriaFieldsBasic.put(KANJI_READING, "Reading");
			criteriaFieldsBasic.put(KANJI_SKIP, "Skip code");
			criteriaFieldsBasic.put(KANJI_STROKE_COUNT, "Stroke count");
			listMessages.put(LIST_CRITERIA_FIELD_BASIC, criteriaFieldsBasic);

			Map<String, String> criteriaFieldsExt = new HashMap<String, String>();
			criteriaFieldsExt.put(KANJI_CLASSIC_RADICAL, "Classic radical");
			criteriaFieldsExt.put(KANJI_CLASSIC_NELSON, "Nelson radical");
			criteriaFieldsExt.put(KANJI_DE_ROO, "De Roo code");
			criteriaFieldsExt.put(KANJI_DIC_NAME, "Dictionary name");
			criteriaFieldsExt.put(KANJI_DIC_INDEX, "Dictionary index");
			criteriaFieldsExt.put(KANJI_FOUR_CORNER, "Four corner code");
			criteriaFieldsExt.put(KANJI_FREQUENCY, "Frequency");
			criteriaFieldsExt.put(KANJI_GRADE, "Grade");
			criteriaFieldsExt.put(KANJI_GRAPH, "Kanji asterisk graph description");
			criteriaFieldsExt.put(KANJI_JIS_CHARSET, "JIS charset");
			criteriaFieldsExt.put(KANJI_JIS_CODE, "JIS kuten code");
			criteriaFieldsExt.put(KANJI_JLPT_LEVEL, "JLPT level");
			criteriaFieldsExt.put(KANJI_MEANING, "Meaning");
			criteriaFieldsExt.put(KANJI_READING_TYPE, "Reading type");
			criteriaFieldsExt.put(KANJI_READING, "Reading");
			criteriaFieldsExt.put(KANJI_SKIP, "Skip code");
			criteriaFieldsExt.put(KANJI_STROKE_COUNT, "Stroke count");
			criteriaFieldsExt.put(KANJI_SPAHN_HADAMITZKY, "Spahn-Hadamitky code");
			criteriaFieldsExt.put(KANJI_UNICODE_VALUE, "Unicode");
			criteriaFieldsExt.put(KANJI_VARIANT_TYPE, "Variant type");
			criteriaFieldsExt.put(KANJI_VARIANT_INDEX, "Variant reference");
			listMessages.put(LIST_CRITERIA_FIELD_EXTENDED, criteriaFieldsExt);

			Map<String, String> criteriaCasesString = new HashMap<String, String>();
			criteriaCasesString.put(CASE_EQUALS, "equal to");
			criteriaCasesString.put(CASE_NOT_EQUALS, "not equal to");
			criteriaCasesString.put(CASE_NULL, "null");
			listMessages.put(LIST_CRITERIA_CASES_STRING, criteriaCasesString);

			Map<String, String> criteriaCasesInteger = new HashMap<String, String>();
			criteriaCasesInteger.put(CASE_GREATHER, "greather than");
			criteriaCasesInteger.put(CASE_EQUALS_OR_GREATHER, "equal or greather than");
			criteriaCasesInteger.put(CASE_LESS, "less than");
			criteriaCasesInteger.put(CASE_EQUALS_OR_LESS, "equal or less than");
			criteriaCasesInteger.put(CASE_EQUALS, "equal to");
			criteriaCasesInteger.put(CASE_NOT_EQUALS, "not equal to");
			criteriaCasesInteger.put(CASE_NULL, "null");
			listMessages.put(LIST_CRITERIA_CASES_INT, criteriaCasesInteger);

			Map<String, String> criteriaValuesJis = new HashMap<String, String>();
			criteriaValuesJis.put(JIS_208, "JIS X 208");
			criteriaValuesJis.put(JIS_212, "JIS X 212");
			criteriaValuesJis.put(JIS_213, "JIS X 213");
			listMessages.put(LIST_CRITERIA_VALUES_JIS, criteriaValuesJis);
			
			Map<String, String> criteriaValuesDic = new HashMap<String, String>();
			criteriaValuesDic.put(DIC_nelson_c,"Modern Reader’s Japanese-English Character Dictionary");
			criteriaValuesDic.put(DIC_nelson_n,"The New Nelson Japanese-English Character Dictionary");
			criteriaValuesDic.put(DIC_halpern_njecd,"New Japanese-English Character Dictionary");
			criteriaValuesDic.put(DIC_halpern_kkld,"Kanji Learners Dictionary");
			criteriaValuesDic.put(DIC_heisig,"Remembering The  Kanji");
			criteriaValuesDic.put(DIC_gakken, "New Dictionary of Kanji Usage");
			criteriaValuesDic.put(DIC_oneill_names, "Japanese Names");
			criteriaValuesDic.put(DIC_oneill_kk, "Essential Kanji");
			criteriaValuesDic.put(DIC_moro, "Daikanwajiten");
			criteriaValuesDic.put(DIC_henshall, "A Guide To Remembering Japanese Characters");
			criteriaValuesDic.put(DIC_sh_kk, "Kanji and Kana");
			criteriaValuesDic.put(DIC_sakade, "A Guide To Reading and Writing Japanese");
			criteriaValuesDic.put(DIC_jf_cards, "Japanese Kanji Flashcards");
			criteriaValuesDic.put(DIC_henshall3, "A Guide To Reading and Writing Japanese");
			criteriaValuesDic.put(DIC_tutt_cards, "Tuttle Kanji Cards");
			criteriaValuesDic.put(DIC_crowley, "The Kanji Way to Japanese Language Power");
			criteriaValuesDic.put(DIC_kanji_in_context, "Kanji in Context");
			criteriaValuesDic.put(DIC_busy_people, "Japanese For Busy People");
			criteriaValuesDic.put(DIC_kodansha_compact, "Kodansha Compact Kanji Guide");
			criteriaValuesDic.put(DIC_maniette, "Les Kanjis dans la tete");
			criteriaValuesDic.put(DIC_kanji_basic_book, "Kanji Basic Book");
			listMessages.put(LIST_CRITERIA_VALUES_DIC,criteriaValuesDic);
			
			Map<String, String> criteriaValuesReadingTypeBasic = new HashMap<String, String>();
			criteriaValuesReadingTypeBasic.put(READ_JA_ON, "on-yomi");
			criteriaValuesReadingTypeBasic.put(READ_JA_KUN, "kun-yomi");
			listMessages.put(LIST_CRITERIA_VALUES_READING_BASIC, criteriaValuesReadingTypeBasic);
			
			Map<String, String> criteriaValuesReadingTypeExt = new HashMap<String, String>();
			criteriaValuesReadingTypeExt.put(READ_JA_ON, "on-yomi");
			criteriaValuesReadingTypeExt.put(READ_JA_KUN, "kun-yomi");
			criteriaValuesReadingTypeExt.put(READ_PINYIN, "pinyin");
			criteriaValuesReadingTypeExt.put(READ_KOREAN_H, "korean hangul");
			criteriaValuesReadingTypeExt.put(READ_KOREAN_R, "korean romanized");
			criteriaValuesReadingTypeExt.put(READ_NANORI, "nanori");
			listMessages.put(LIST_CRITERIA_VALUES_READING_EXTENDED, criteriaValuesReadingTypeExt);

			Map<String, String> languagesSupported = new HashMap<String, String>();
			languagesSupported.put(LANGUAJE_EN, "English");
			languagesSupported.put(LANGUAJE_ES, "Spanish");
			listMessages.put(LIST_LANGUAGES_SUPPORTED, languagesSupported);

			// ORDENABLE PROPERTIES

			Map<String, String> ordenableFields = new HashMap<String, String>();

			ordenableFields.put(ORDER_UNICODE_VALUE, "Unicode");
			ordenableFields.put(ORDER_JIS_CHARSET, "JIS charset");
			ordenableFields.put(ORDER_JIS_CODE, "JIS code");
			ordenableFields.put(ORDER_CLASSIC_NELSON, "Nelson Radical");
			ordenableFields.put(ORDER_CLASSIC_RADICAL, "Classic Radical");
			ordenableFields.put(ORDER_GRADE, "Grade");
			ordenableFields.put(ORDER_STROKE_COUNT, "Stroke Number");
			ordenableFields.put(ORDER_FREQUENCY, "Frequency");
			ordenableFields.put(ORDER_SKIP, "SKIP Code");
			ordenableFields.put(ORDER_SPAHN_HADAMITZKY, "Spahn-Hadamitzky code");
			ordenableFields.put(ORDER_FOUR_CORNER, "Four Corner Code");
			ordenableFields.put(ORDER_DE_ROO, "De Roo COde");
			ordenableFields.put(ORDER_GRAPH, "Asterisk Graph");
			ordenableFields.put(ORDER_DIC_INDEX, "Dictionary Reference");
			ordenableFields.put(ORDER_MEANING, "First Meaning");
			ordenableFields.put(ORDER_READING, "First Reading");
			listMessages.put(LIST_ORDENABLE_FIELDS, ordenableFields);

			Map<String, String> ordenableMethodsStr = new HashMap<String, String>();
			ordenableMethodsStr.put(ORDERMETHODSSTR_ALPHABETICALLY, "Alphabetically");
			ordenableMethodsStr.put(ORDERMETHODSSTR_ALPHABETICALLY_INVERSE, "Alphabetically inverse");
			ordenableMethodsStr.put(ORDERMETHODSSTR_RAMDOM, "Ramdom");
			listMessages.put(LIST_ORDENABLE_METHODS_STRING, ordenableMethodsStr);

			Map<String, String> ordenableMethodsInt = new HashMap<String, String>();
			ordenableMethodsInt.put(ORDERMETHODSINT_FROM_LEAST_TO_GREATEST, "From least to greatest");
			ordenableMethodsInt.put(ORDERMETHODSINT_FROM_GREATEST_TO_LEAST, "From greatest to least");
			ordenableMethodsInt.put(ORDERMETHODSINT_RAMDOM, "Ramdom");
			listMessages.put(LIST_ORDENABLE_METHODS_INTEGER, ordenableMethodsInt);

			// RADICAL TYPES

			messages.put(RADICAL_TYPE_original, "Original");
			messages.put(RADICAL_TYPE_top, "Top");
			messages.put(RADICAL_TYPE_bottom, "Bottom");
			messages.put(RADICAL_TYPE_left, "Left");
			messages.put(RADICAL_TYPE_right, "Right");
			messages.put(RADICAL_TYPE_enclosure, "Enclosure");
			messages.put(RADICAL_TYPE_variable, "Variable");

			// MainWindow
				// Window name
			
			messages.put(WINMAIN_name, "JavaDiKt, the definitive Kanji dictionary");

				// Option tab names
			messages.put(WINMAIN_TAB_CONTROL_About, "More");
			messages.put(WINMAIN_TAB_CONTROL_Criteria, "Criteria");
			messages.put(WINMAIN_TAB_CONTROL_Drawing, "Strokes");
			messages.put(WINMAIN_TAB_CONTROL_Export, "Export");
			messages.put(WINMAIN_TAB_CONTROL_Config, "Options");

				// Border texts
			messages.put(WINMAIN_CRITERIA_BorderTextQueryData, "Search list");
			messages.put(WINMAIN_CRITERIA_BorderTextQueryMaker, "Find kanji where it's");
			messages.put(WINMAIN_CRITERIA_BorderTextQueryControls, "Modify current search");
			messages.put(WINMAIN_RESULTS_BorderTextResults, "Results");
			messages.put(WINMAIN_STROKES_BorderTextControls, "Modify");
			messages.put(WINMAIN_STROKES_BorderTextAndOr, "Add stroke search");
			messages.put(WINMAIN_RESULTS_EXPORT_BorderTextControls, "Modify selection");
			messages.put(WINMAIN_EXPORT_BorderTextExportAs, "1.- Export as");
			messages.put(WINMAIN_EXPORT_BorderTextExportSorting, "2.- Sort as");
			messages.put(WINMAIN_EXPORT_BorderTextExportExtra, "3.- Extra config.");

				// Query column names
			messages.put(WINMAIN_QUERY_DATA_columnAndOr, "And/Or");
			messages.put(WINMAIN_QUERY_DATA_columnField, "Field");
			messages.put(WINMAIN_QUERY_DATA_columnThatFulfilsThat, "Fulfils");
			messages.put(WINMAIN_QUERY_DATA_columnValue, "Value");
			messages.put(WINMAIN_EXPORT_SORTING_columnLevel, "Level");
			messages.put(WINMAIN_EXPORT_SORTING_columnOrderBy, "Order by");
			messages.put(WINMAIN_EXPORT_SORTING_columnField, "Mode");
			
				//Query table cell string
			
			messages.put(WINMAIN_QUERY_DATA_cellAndString, "And");
			messages.put(WINMAIN_QUERY_DATA_cellOrString, "Or");
			messages.put(WINMAIN_QUERY_DATA_cellFirstString, "-");
			
				// Text Label
			messages.put(WINMAIN_CRITERIA_LabelFieldComboBox, "Where");
			messages.put(WINMAIN_CRITERIA_LabelFulfilsComboBox, "is");
			messages.put(WINMAIN_CRITERIA_LabelValueComboBox, "");
			messages.put(WINMAIN_EXPORT_LabelFormatComboBox, "Format:");
			messages.put(WINMAIN_EXPORT_LabelStyleComboBox, "Style:");
			messages.put(WINMAIN_EXPORT_LabelDirButton, "Path:");
			messages.put(WINMAIN_OPTIONS_LabelLanguajeComboBox, "Languaje:");
			messages.put(WINMAIN_OPTIONS_LabelKanjiFileTextField, "Kanji file path:");
			messages.put(WINMAIN_OPTIONS_LabelStrokeFileTextField, "Stroke file path:");
			messages.put(WINMAIN_OPTIONS_LabelExtendedInfoCheckBox, "Include extended information in kanji info dialog");
			messages.put(WINMAIN_OPTIONS_LabelExtendedKanjiListCheckBox, "Find in extended kanji list (>13000 kanjis)");
			messages.put(WINMAIN_OPTIONS_LabelRomajiCheckBox, "Kana texts in romaji");
			
				// Button text 
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerAndText, "And");
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerAddText, "Add");
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerChangeText, "Change");
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerOrText, "Or");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlResetText,"Reset");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlRemoveText, "Remove entries");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlModifyText, "Modify");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlExecuteText, "Find");
			messages.put(WINMAIN_STROKES_ButtonDrawPanelReset, "Reset draw");
			messages.put(WINMAIN_STROKES_ButtonDrawPanelRemoveLastStroke, "Remove last stroke");
			messages.put(WINMAIN_STROKES_ButtonDrawPanelRemoveLastLine, "Remove last line");
			messages.put(WINMAIN_RESULTS_ButtonExportAll,"All");
			messages.put(WINMAIN_RESULTS_ButtonExportInvert, "Invert");
			messages.put(WINMAIN_RESULTS_ButtonExportReset, "Reset");
			messages.put(WINMAIN_EXPORT_ButtonExportExtraOptions, "Extra Options");
			messages.put(WINMAIN_EXPORT_ButtonSortingAdd, "Add");
			messages.put(WINMAIN_EXPORT_ButtonSortingReset, "Reset");
			messages.put(WINMAIN_EXPORT_ButtonSortingRemove, "Remove");
			messages.put(WINMAIN_EXPORT_ButtonSortingModify, "Modify");
			messages.put(WINMAIN_EXPORT_ButtonExtraRearrange, "Rearrange");
			messages.put(WINMAIN_EXPORT_ButtonExtraOptions, "More...");
			messages.put(WINMAIN_EXPORT_ButtonExport, "Export");
			messages.put(WINMAIN_OPTIONS_ButtonSave, "Save");;
			messages.put(WINMAIN_OPTIONS_ButtonAbout, "About");
			
			//Export Sorter Adder Dialog 
				//Window name
			messages.put(WINEXPORTRESORTERADDER_name, "Add automatic sorter");
				//Buttons
			messages.put(WINEXPORTRESORTERADDER_ButtonAdd, "Add/Change");
			messages.put(WINEXPORTRESORTERADDER_ButtonCancel, "Finish");
				// Labels
			messages.put(WINEXPORTRESORTERADDER_LabelLevel, "Level");
			messages.put(WINEXPORTRESORTERADDER_LabelOrderBy, "Order by");
			messages.put(WINEXPORTRESORTERADDER_LabelField, "Field");
			
			// Export Rearranger Dialog
				// Window name
			messages.put(WINEXPORTREARRANGER_name, "Rearrange selection");
				// Button
			messages.put(WINEXPORTREARRANGER_acceptButton, "Accept");
			messages.put(WINEXPORTREARRANGER_cancelButton, "Cancel");
				// table column names
			messages.put(WINEXPORTREARRANGER_kanjiLiteral, "Kanji");
				// Special kanji field names
			messages.put(KANJI_FIRST_MEANING, "First meaning");
			messages.put(KANJI_FIRST_READING, "First %s reading");
			messages.put(KANJI_DIC_REFERENCE, "\"%s\" dictionary reference ");

			// Kanji info window
				// Panel titles

			messages.put(WINKANJIINFO_PanelTitle_Radical, "Radical");
			messages.put(WINKANJIINFO_PanelTitle_StrokeCount, "Stroke count");
			messages.put(WINKANJIINFO_PanelTitle_Grade, "Gradde");
			messages.put(WINKANJIINFO_PanelTitle_JLPTLevel, "JLPT Level");
			messages.put(WINKANJIINFO_PanelTitle_Frequency, "Frequency");
			messages.put(WINKANJIINFO_PanelTitle_NelsonRadical, "Nelson Rad.");
			messages.put(WINKANJIINFO_PanelTitle_Unicode, "Unicode");
			messages.put(WINKANJIINFO_PanelTitle_JIS, "JIS kuten");
			messages.put(WINKANJIINFO_PanelTitle_DicReferences, "Dictionaire references of this kanji");
			messages.put(WINKANJIINFO_PanelTitle_Graph, "Asterisk graph info");
			messages.put(WINKANJIINFO_PanelTitle_Meanings, "Meanings");
			messages.put(WINKANJIINFO_PanelTitle_Readings, "Readings");
			messages.put(WINKANJIINFO_PanelTitle_Variants, "Variants");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes, "Query codes");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_SKIP, "SKIP code");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_FourCorner, "Four corner code");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_SpahnHadamitzky, "Spahn-Hadamiztky code");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_DeRoo, "De Roo code");

			// Radical Info panel
			messages.put(PANELRADICALINFO_radicalNumber, "Number");
			messages.put(PANELRADICALINFO_radicalType, "Type");
			messages.put(PANELRADICALINFO_radicalNames, "Names");
			messages.put(PANELRADICALINFO_radicalUnicode, "Unicode");
			messages.put(PANELRADICALINFO_radicalUnicodeName, "Unicode name");
			messages.put(PANELRADICALINFO_radicalKangxi, "Kangxi unicode");

			// INFO DIALOG
				// title
			messages.put(INFODIALOG_title_error, "Error");
			messages.put(INFODIALOG_title_info, "Info");
				// Messages
			messages.put(INFODIALOG_buttonAccept, "Accept");
			messages.put(INFODIALOG_errorMessage_castException, "Please, insert a number");
			messages.put(INFODIALOG_errorMessage_exportFail, "Error when exporting: %");
			messages.put(INFODIALOG_infoMessage_exportSuccess, "Export fisnished succesfully.");
			messages.put(INFODIALOG_infoMessage_functionalityNotAvaible, "Functionality not avaible yet.");

			// ABOUT WINDOW
				// Title
			messages.put(ABOUTWIN_title, "About JavaDiKt");
			messages.put(ABOUTWIN_javadicText, "<html><b>JavaDiKt %s</b> <i>%s</i> with %s %s<br>©2010, Luis Alfonso Arce Gonzalez<br><br>Licensed under <a href=\"http://www.gnu.org/licenses/gpl.html\">GNU General Public License version 3<a>.<br><br>You can find tips, guides and information about newer versions in JavaDiKt's webpage <a href=\"http://www.javadikt.net\">JavaDiKt.net</a></html>");
			messages.put(ABOUTWIN_borderTextLicense, "License");
			messages.put(ABOUTWIN_licenseText, "<html><font size=-2>JavaDiKt wouldn't be possible without the efforts and kindness of people who developed the following resources and libraries,people who grants without any cost use permission of their work under the licences below:<br><br><b><a href=\"http://www.csse.monash.edu.au/~jwb/kanjidic.html\">KANJIDIC</a></b>, by Jim Breen of the <i><a href=\"http://www.edrdg.org/\">ELECTRONIC DICTIONARY RESEARCH AND DEVELOPMENT GROUP</a></i>, under <i><a href=\"http://creativecommons.org/licenses/by-sa/3.0/\">Creative Commons Attribution-ShareAlike3.0 Unported</a></i> license .<br><br><b><a href=\"http://www.kanji.org/kanji/dictionaries/skip_permission.htm\">SYSTEM OF KANJI INDEXING BY PATTERNS(SKIP)</a></b>, by Jack Halpern of the <i><a href=\"http://www.kanji.org\">Kanji Dictionary Publishing Society</a></i> under <i><a href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\">Creative CommonsAttribution-Noncommercial-Share Alike 3.0 Unported</a></i> license.<br><br><b><a href=\"http://www.neodatis.org/\">Neodatis ODB</a></b>, library under <a href=\"http://www.gnu.org/licenses/lgpl-2.1.html\">LGPL 2.1</a> license.<br><br><b><a href=\"http://www.xom.nu/\">XOM</a></b>, library under <a href=\"http://www.gnu.org/licenses/lgpl-2.1.html\">LGPL 2.1</a> license.</font></html>");
			initialized = true;
			buildKeyMap();

		} else if(languaje.toUpperCase().equals("ES")){
			
			LENGUAGE = "ES";
			
			if (initialized = true) {
				messages = new HashMap<String, String>();
				listMessages = new HashMap<String, Map<String, String>>();
			}
			
			// generic

			messages.put(GEN_DIR_FILES_READ_ERROR_C, "**Error al acceder a los archivos en la carpeta \"%s\", puede que no exista, esté bloqueado o no tenga privilegios de lectura suficientes.");
			messages.put(GEN_GENERIC_READ_ERROR_C_colon, "**No se ha podido leer el archivo o carpeta \"%s\": ");
			messages.put(GEN_EXIT_SUCESS_C, "El archivo de sincronización se creó satisfactoriaente en \"%s\".");
			messages.put(GEN_EXIT_SUCCES_TIME_C, "Tiempo de ejecución:[%s:%s:%s]");
			messages.put(GEN_INDEXING, "[Indexando...]\n");
			messages.put(GEN_LENGUAJE_DOES_NOT_EXISTS_C, "**El lenguaje especificado %s no existe o no está definido, inicializando configuración de idioma por defecto.");
			messages.put(GEN_LOCAL_LABEL, "Introduzca la ruta completa del directorio al que se le realizará el archivo local de sincronización y la ruta completa y el nombre del archivo de sincronización.");
			messages.put(GEN_NAME_ERROR, "**El nombre de archivo introducido no es válido.");
			messages.put(GEN_NAME_OR_PATH_ERROR, "**El nombre o ruta de archivo introducido no es válido.");
			messages.put(GEN_NOT_ISO_639_1, "**El lenguaje introducido no tiene formato ISO-639-1, inicializando configuración de idioma por defecto.");
			messages.put(GEN_NOT_MD5_HASH, "No es un hash MD5.");
			messages.put(GEN_NULL_POINTER, "**Error de puntero nulo.");
			messages.put(GEN_PATH_DOES_NOT_EXISTS, "**El directorio introducido no existe.");
			messages.put(GEN_PATH_ERROR, "**El nombre de directorio introducido no es válido.");
			messages.put(GEN_READ_ERROR_C, "**No se puede leer el archivo o carpeta \"%s\", puede que no exista, esté bloqueado o no tenga privilegios de lectura suficientes.");
			messages.put(GEN_RECOVERING_FILE_SYSTEM, "\n[Recuperando sistema de archivos...]\n");
			messages.put(GEN_UNKNOWN_ERROR, "**Se ha producido un error inesperado.");
			messages.put(GEN_WRITING, "\n[Escribiendo archivo de sincronización...]");

			// FILE DESCRIPTIONS

			messages.put(FILE_DESCRIPTION_JDK_KANJI_FILE, "Archivo .jdk de kanjis de JavaDiKt ");
			messages.put(FILE_DESCRIPTION_TXT_STROKE_FILE, "Archivo .txt de trazos de JavaDiKt ");

			// PROGRAM LISTS

			Map<String, String> criteriaFieldsBasic = new HashMap<String, String>();
			criteriaFieldsBasic.put(KANJI_CLASSIC_RADICAL,"Radical");
			criteriaFieldsBasic.put(KANJI_FREQUENCY,"Frecuencia");
			criteriaFieldsBasic.put(KANJI_DIC_NAME,"Nombre del Diccionario");
			criteriaFieldsBasic.put(KANJI_DIC_INDEX,"Índice del diccionario");
			criteriaFieldsBasic.put(KANJI_GRADE,"Grado");
			criteriaFieldsBasic.put(KANJI_JLPT_LEVEL,"Nivel JLPT");
			criteriaFieldsBasic.put(KANJI_MEANING,"Significado");
			criteriaFieldsBasic.put(KANJI_READING_TYPE,"Tipo de lectura");
			criteriaFieldsBasic.put(KANJI_READING,"Lectura");
			criteriaFieldsBasic.put(KANJI_SKIP,"Código SKIP");
			criteriaFieldsBasic.put(KANJI_STROKE_COUNT, "Número de trazos");
			listMessages.put(LIST_CRITERIA_FIELD_BASIC, criteriaFieldsBasic);

			Map<String, String> criteriaFieldsExt = new HashMap<String, String>();
			criteriaFieldsExt.put(KANJI_CLASSIC_RADICAL, "Radical clásico");
			criteriaFieldsExt.put(KANJI_CLASSIC_NELSON, "Radical Nelson");
			criteriaFieldsExt.put(KANJI_DE_ROO, "Código de Roo");
			criteriaFieldsExt.put(KANJI_DIC_NAME, "Nombre del diccionario");
			criteriaFieldsExt.put(KANJI_DIC_INDEX, "Índice del diccionario");
			criteriaFieldsExt.put(KANJI_FOUR_CORNER, "Código cuatro esquinas");
			criteriaFieldsExt.put(KANJI_FREQUENCY, "Frecuencia");
			criteriaFieldsExt.put(KANJI_GRADE, "Grado");
			criteriaFieldsExt.put(KANJI_GRAPH, "Grafo asterisco del kanji");
			criteriaFieldsExt.put(KANJI_JIS_CHARSET, "Juego de caracteres JIS");
			criteriaFieldsExt.put(KANJI_JIS_CODE, "Código JIS kuten");
			criteriaFieldsExt.put(KANJI_JLPT_LEVEL, "Nivel JLPT");
			criteriaFieldsExt.put(KANJI_MEANING, "Significado");
			criteriaFieldsExt.put(KANJI_READING_TYPE, "Tipo de lectura");
			criteriaFieldsExt.put(KANJI_READING, "Lectura");
			criteriaFieldsExt.put(KANJI_SKIP, "Código SKIP");
			criteriaFieldsExt.put(KANJI_STROKE_COUNT, "Número de trazos");
			criteriaFieldsExt.put(KANJI_SPAHN_HADAMITZKY, "Código Spahn-Hadamitzky");
			criteriaFieldsExt.put(KANJI_UNICODE_VALUE, "Unicode");
			criteriaFieldsExt.put(KANJI_VARIANT_TYPE, "Tipo de variante");
			criteriaFieldsExt.put(KANJI_VARIANT_INDEX, "Índice de la variante");
			listMessages.put(LIST_CRITERIA_FIELD_EXTENDED, criteriaFieldsExt);

			Map<String, String> criteriaCasesString = new HashMap<String, String>();
			criteriaCasesString.put(CASE_EQUALS, "igual que");
			criteriaCasesString.put(CASE_NOT_EQUALS, "distinto que");
			criteriaCasesString.put(CASE_NULL, "nulo");
			listMessages.put(LIST_CRITERIA_CASES_STRING, criteriaCasesString);

			Map<String, String> criteriaCasesInteger = new HashMap<String, String>();
			criteriaCasesInteger.put(CASE_GREATHER, "mayor que");
			criteriaCasesInteger.put(CASE_EQUALS_OR_GREATHER, "mayor o igual que");
			criteriaCasesInteger.put(CASE_LESS, "menor que");
			criteriaCasesInteger.put(CASE_EQUALS_OR_LESS, "menor o igual que");
			criteriaCasesInteger.put(CASE_EQUALS, "igual que");
			criteriaCasesInteger.put(CASE_NOT_EQUALS, "distinto que");
			criteriaCasesInteger.put(CASE_NULL, "nulo");
			listMessages.put(LIST_CRITERIA_CASES_INT, criteriaCasesInteger);

			Map<String, String> criteriaValuesJis = new HashMap<String, String>();
			criteriaValuesJis.put(JIS_208, "JIS X 208");
			criteriaValuesJis.put(JIS_212, "JIS X 212");
			criteriaValuesJis.put(JIS_213, "JIS X 213");
			listMessages.put(LIST_CRITERIA_VALUES_JIS, criteriaValuesJis);

			Map<String, String> criteriaValuesDic = new HashMap<String, String>();
			criteriaValuesDic.put(DIC_nelson_c, "Modern Reader’s Japanese-English Character Dictionary");
			criteriaValuesDic.put(DIC_nelson_n, "The New Nelson Japanese-English Character Dictionary");
			criteriaValuesDic.put(DIC_halpern_njecd, "New Japanese-English Character Dictionary");
			criteriaValuesDic.put(DIC_halpern_kkld, "Kanji Learners Dictionary");
			criteriaValuesDic.put(DIC_heisig, "Remembering The  Kanji");
			criteriaValuesDic.put(DIC_gakken, "New Dictionary of Kanji Usage");
			criteriaValuesDic.put(DIC_oneill_names, "Japanese Names");
			criteriaValuesDic.put(DIC_oneill_kk, "Essential Kanji");
			criteriaValuesDic.put(DIC_moro, "Daikanwajiten");
			criteriaValuesDic.put(DIC_henshall, "A Guide To Remembering Japanese Characters");
			criteriaValuesDic.put(DIC_sh_kk, "Kanji and Kana");
			criteriaValuesDic.put(DIC_sakade, "A Guide To Reading and Writing Japanese");
			criteriaValuesDic.put(DIC_jf_cards, "Japanese Kanji Flashcards");
			criteriaValuesDic.put(DIC_henshall3, "A Guide To Reading and Writing Japanese");
			criteriaValuesDic.put(DIC_tutt_cards, "Tuttle Kanji Cards");
			criteriaValuesDic.put(DIC_crowley, "The Kanji Way to Japanese Language Power");
			criteriaValuesDic.put(DIC_kanji_in_context, "Kanji in Context");
			criteriaValuesDic.put(DIC_busy_people, "Japanese For Busy People");
			criteriaValuesDic.put(DIC_kodansha_compact, "Kodansha Compact Kanji Guide");
			criteriaValuesDic.put(DIC_maniette, "Les Kanjis dans la tete");
			criteriaValuesDic.put(DIC_kanji_basic_book, "Kanji Basic Book");
			listMessages.put(LIST_CRITERIA_VALUES_DIC, criteriaValuesDic);

			Map<String, String> criteriaValuesReadingTypeBasic = new HashMap<String, String>();
			criteriaValuesReadingTypeBasic.put(READ_JA_ON, "on-yomi");
			criteriaValuesReadingTypeBasic.put(READ_JA_KUN, "kun-yomi");
			listMessages.put(LIST_CRITERIA_VALUES_READING_BASIC, criteriaValuesReadingTypeBasic);

			Map<String, String> criteriaValuesReadingTypeExt = new HashMap<String, String>();
			criteriaValuesReadingTypeExt.put(READ_JA_ON, "on-yomi");
			criteriaValuesReadingTypeExt.put(READ_JA_KUN, "kun-yomi");
			criteriaValuesReadingTypeExt.put(READ_PINYIN, "pinyin");
			criteriaValuesReadingTypeExt.put(READ_KOREAN_H, "coreano hangul");
			criteriaValuesReadingTypeExt.put(READ_KOREAN_R, "coreano romanizado");
			criteriaValuesReadingTypeExt.put(READ_NANORI, "nanori");
			listMessages.put(LIST_CRITERIA_VALUES_READING_EXTENDED, criteriaValuesReadingTypeExt);

			Map<String, String> languagesSupported = new HashMap<String, String>();
			languagesSupported.put(LANGUAJE_EN, "Inglés");
			languagesSupported.put(LANGUAJE_ES, "Español");
			listMessages.put(LIST_LANGUAGES_SUPPORTED, languagesSupported);

			Map<String, String> ordenableFields = new HashMap<String, String>();

			ordenableFields.put(ORDER_UNICODE_VALUE, "Código unicode");
			ordenableFields.put(ORDER_JIS_CHARSET, "Juego de caracteres JIS");
			ordenableFields.put(ORDER_JIS_CODE, "Código JIS");
			ordenableFields.put(ORDER_CLASSIC_NELSON, "Radical Nelson");
			ordenableFields.put(ORDER_CLASSIC_RADICAL, "Radical Clasico");
			ordenableFields.put(ORDER_GRADE, "Grado");
			ordenableFields.put(ORDER_STROKE_COUNT, "Número de trazos");
			ordenableFields.put(ORDER_FREQUENCY, "Frecuencia");
			ordenableFields.put(ORDER_SKIP, "Código SKIP");
			ordenableFields.put(ORDER_SPAHN_HADAMITZKY, "Código Spahn-Hadamitzky");
			ordenableFields.put(ORDER_FOUR_CORNER, "Código cuatro esquinas");
			ordenableFields.put(ORDER_DE_ROO, "Código de Roo");
			ordenableFields.put(ORDER_GRAPH, "Grafo asterisco");
			ordenableFields.put(ORDER_DIC_INDEX, "Referencia en diccionario");
			ordenableFields.put(ORDER_MEANING, "Primer significado");
			ordenableFields.put(ORDER_READING, "Primera lectura");
			listMessages.put(LIST_ORDENABLE_FIELDS, ordenableFields);

			Map<String, String> ordenableMethodsStr = new HashMap<String, String>();
			ordenableMethodsStr.put(ORDERMETHODSSTR_ALPHABETICALLY, "Alfabético");
			ordenableMethodsStr.put(ORDERMETHODSSTR_ALPHABETICALLY_INVERSE, "Alfabéticamente inverso");
			ordenableMethodsStr.put(ORDERMETHODSSTR_RAMDOM, "Aleatorio");
			listMessages.put(LIST_ORDENABLE_METHODS_STRING, ordenableMethodsStr);

			Map<String, String> ordenableMethodsInt = new HashMap<String, String>();
			ordenableMethodsInt.put(ORDERMETHODSINT_FROM_LEAST_TO_GREATEST, "De menor a mayor");
			ordenableMethodsInt.put(ORDERMETHODSINT_FROM_GREATEST_TO_LEAST, "De mayor a menor");
			ordenableMethodsInt.put(ORDERMETHODSINT_RAMDOM, "Aleatorio");
			listMessages.put(LIST_ORDENABLE_METHODS_INTEGER, ordenableMethodsInt);

			// RADICAL TYPES

			messages.put(RADICAL_TYPE_original, "Original");
			messages.put(RADICAL_TYPE_top, "Arriba");
			messages.put(RADICAL_TYPE_bottom, "Abajo");
			messages.put(RADICAL_TYPE_left, "Izquierda");
			messages.put(RADICAL_TYPE_right, "Derecha");
			messages.put(RADICAL_TYPE_enclosure, "Alrededor");
			messages.put(RADICAL_TYPE_variable, "Variable");

			// MainWinfow
				// Window name
			messages.put(WINMAIN_name, "JavaDiKt, el diccionario de kanjis definitivo");
			
				// Option tab names
			messages.put(WINMAIN_TAB_CONTROL_About, "Info");
			messages.put(WINMAIN_TAB_CONTROL_Criteria, "Criterio");
			messages.put(WINMAIN_TAB_CONTROL_Drawing, "Trazos");
			messages.put(WINMAIN_TAB_CONTROL_Export, "Exportar");
			messages.put(WINMAIN_TAB_CONTROL_Config, "Opciones");

				// Border texts
			messages.put(WINMAIN_CRITERIA_BorderTextQueryData, "Lista de búsqueda");
			messages.put(WINMAIN_CRITERIA_BorderTextQueryMaker, "Buscar kanji tal que su");
			messages.put(WINMAIN_CRITERIA_BorderTextQueryControls, "Modificar la búsqueda actual");
			messages.put(WINMAIN_RESULTS_BorderTextResults, "Resultados");
			messages.put(WINMAIN_STROKES_BorderTextControls, "Modificar");
			messages.put(WINMAIN_STROKES_BorderTextAndOr, "Añadir búsqueda");
			messages.put(WINMAIN_RESULTS_EXPORT_BorderTextControls, "Modificar selección");
			messages.put(WINMAIN_EXPORT_BorderTextExportAs, "1.- Exportar como");
			messages.put(WINMAIN_EXPORT_BorderTextExportSorting, "2.- Ordenar por");
			messages.put(WINMAIN_EXPORT_BorderTextExportExtra, "3.- Config. extra");
			
				// Query table column names
			messages.put(WINMAIN_QUERY_DATA_columnAndOr, "Y/Ó");
			messages.put(WINMAIN_QUERY_DATA_columnField, "Campo");
			messages.put(WINMAIN_QUERY_DATA_columnThatFulfilsThat, "Cumple");
			messages.put(WINMAIN_QUERY_DATA_columnValue, "Valor");
			messages.put(WINMAIN_EXPORT_SORTING_columnLevel, "Nivel");
			messages.put(WINMAIN_EXPORT_SORTING_columnOrderBy, "Ordenar por");
			messages.put(WINMAIN_EXPORT_SORTING_columnField, "Modo");
			
				// Query table cell string
			messages.put(WINMAIN_QUERY_DATA_cellAndString, "Y");
			messages.put(WINMAIN_QUERY_DATA_cellOrString, "Ó");
			messages.put(WINMAIN_QUERY_DATA_cellFirstString, "-");

				// Text Label
			messages.put(WINMAIN_CRITERIA_LabelFieldComboBox, "donde");
			messages.put(WINMAIN_CRITERIA_LabelFulfilsComboBox, "sea");
			messages.put(WINMAIN_CRITERIA_LabelValueComboBox, "");
			messages.put(WINMAIN_EXPORT_LabelFormatComboBox, "Formato:");
			messages.put(WINMAIN_EXPORT_LabelStyleComboBox, "Estilo:");
			messages.put(WINMAIN_EXPORT_LabelDirButton, "Ruta:");
			messages.put(WINMAIN_OPTIONS_LabelLanguajeComboBox, "Idioma:");
			messages.put(WINMAIN_OPTIONS_LabelKanjiFileTextField, "Ruta del archivo de kanjis:");
			messages.put(WINMAIN_OPTIONS_LabelStrokeFileTextField, "Ruta del archivo de trazos:");
			messages.put(WINMAIN_OPTIONS_LabelExtendedInfoCheckBox, "<html>Incluir información avanzada en el<br>diálogo de información de kanjis");
			messages.put(WINMAIN_OPTIONS_LabelExtendedKanjiListCheckBox, "<html>Buscar en la lista extendida de<br>kanjis (>13000 kanjis)");
			messages.put(WINMAIN_OPTIONS_LabelRomajiCheckBox, "Texto kana romanizado");

				// Button text
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerAndText, "Y");
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerAddText, "Añadir");
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerChangeText, "Cambiar");
			messages.put(WINMAIN_CRITERIA_ButtonQueryMakerOrText, "Ó");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlResetText, "Reset");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlRemoveText, "Borrar entrada/s");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlModifyText, "Modificar");
			messages.put(WINMAIN_CRITERIA_ButtonQueryControlExecuteText, "Buscar");
			messages.put(WINMAIN_STROKES_ButtonDrawPanelReset, "Borrar dibujo");
			messages.put(WINMAIN_STROKES_ButtonDrawPanelRemoveLastStroke, "Borrar último trazo");
			messages.put(WINMAIN_STROKES_ButtonDrawPanelRemoveLastLine, "Borrar última línea");
			messages.put(WINMAIN_RESULTS_ButtonExportAll, "Todos");
			messages.put(WINMAIN_RESULTS_ButtonExportInvert, "Invertir");
			messages.put(WINMAIN_RESULTS_ButtonExportReset, "Reset");
			messages.put(WINMAIN_EXPORT_ButtonExportExtraOptions, "Opciones Adicionales");
			messages.put(WINMAIN_EXPORT_ButtonSortingAdd, "Añadir");
			messages.put(WINMAIN_EXPORT_ButtonSortingReset, "Reset");
			messages.put(WINMAIN_EXPORT_ButtonSortingRemove, "Borrar");
			messages.put(WINMAIN_EXPORT_ButtonSortingModify, "Modificar");
			messages.put(WINMAIN_EXPORT_ButtonExtraRearrange, "Reordenar");
			messages.put(WINMAIN_EXPORT_ButtonExtraOptions, "Más...");
			messages.put(WINMAIN_EXPORT_ButtonExport, "Exportar");
			messages.put(WINMAIN_OPTIONS_ButtonSave, "Guardar");;
			messages.put(WINMAIN_OPTIONS_ButtonAbout, "Acerca de");

			// Export Sorter Adder Dialog
				// Window name
			messages.put(WINEXPORTRESORTERADDER_name, "Añadir ordenador automático");
				// Buttons
			messages.put(WINEXPORTRESORTERADDER_ButtonAdd, "Añadir/Cambiar");
			messages.put(WINEXPORTRESORTERADDER_ButtonCancel, "Terminar");
				// Labels
			messages.put(WINEXPORTRESORTERADDER_LabelLevel, "Nivel");
			messages.put(WINEXPORTRESORTERADDER_LabelOrderBy, "Ordenar según");
			messages.put(WINEXPORTRESORTERADDER_LabelField, "Propiedad");

			// Export Rearranger Dialog
				// Window name
			messages.put(WINEXPORTREARRANGER_name, "Reordenar selección");
				// Button
			messages.put(WINEXPORTREARRANGER_acceptButton, "Aceptar");
			messages.put(WINEXPORTREARRANGER_cancelButton, "Cancelar");
				// table column names
			messages.put(WINEXPORTREARRANGER_kanjiLiteral, "Kanji");
				// Special kanji field names
			messages.put(KANJI_FIRST_MEANING, "Primer significado");
			messages.put(KANJI_FIRST_READING, "Primera lectura %s");
			messages.put(KANJI_DIC_REFERENCE, "Referencia en diccionario \"%s\"");

			// Kanji info window
				// Panel titles

			messages.put(WINKANJIINFO_PanelTitle_Radical, "Radical");
			messages.put(WINKANJIINFO_PanelTitle_StrokeCount, "Nº de trazos");
			messages.put(WINKANJIINFO_PanelTitle_Grade, "Grado");
			messages.put(WINKANJIINFO_PanelTitle_JLPTLevel, "Nivel JLPT");
			messages.put(WINKANJIINFO_PanelTitle_Frequency, "Frecuencia");
			messages.put(WINKANJIINFO_PanelTitle_NelsonRadical, "Rad. Nelson");
			messages.put(WINKANJIINFO_PanelTitle_Unicode, "Unicode");
			messages.put(WINKANJIINFO_PanelTitle_JIS, "JIS kuten");
			messages.put(WINKANJIINFO_PanelTitle_DicReferences, "Referencias de este kanji en diccionarios");
			messages.put(WINKANJIINFO_PanelTitle_Graph, "Información gráfica de astrisco");
			messages.put(WINKANJIINFO_PanelTitle_Meanings, "Significados");
			messages.put(WINKANJIINFO_PanelTitle_Readings, "Lecturas");
			messages.put(WINKANJIINFO_PanelTitle_Variants, "Variantes");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes, "Códigos de búsqueda");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_SKIP, "SKIP");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_FourCorner, "Cuatro esquinas");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_SpahnHadamitzky, "Spahn-Hadamiztky");
			messages.put(WINKANJIINFO_PanelTitle_QueryCodes_DeRoo, "De Roo");

			// Radical Info panel
			messages.put(PANELRADICALINFO_radicalNumber, "Número");
			messages.put(PANELRADICALINFO_radicalType, "Tipo");
			messages.put(PANELRADICALINFO_radicalNames, "Nombres");
			messages.put(PANELRADICALINFO_radicalUnicode, "Unicode");
			messages.put(PANELRADICALINFO_radicalUnicodeName, "Nombre unicode");
			messages.put(PANELRADICALINFO_radicalKangxi, "Unicode kangxi");

			// INFO DIALOG
				// title
			messages.put(INFODIALOG_title_error, "Error");
			messages.put(INFODIALOG_title_info, "Info");
				// Messages
			messages.put(INFODIALOG_buttonAccept, "Aceptar");
			messages.put(INFODIALOG_errorMessage_castException, "Inserte un número");
			messages.put(INFODIALOG_errorMessage_exportFail, "Error al exportar: %s");
			messages.put(INFODIALOG_infoMessage_exportSuccess, "Exportación finalizada con éxito");
			messages.put(INFODIALOG_infoMessage_functionalityNotAvaible, "Funcionalidad no implementada aún.");

			// ABOUT WINDOW
				// Title
			messages.put(ABOUTWIN_title, "Acerca de JavaDiKt");
			messages.put(ABOUTWIN_javadicText, "<html><b>JavaDiKt %s</b> <i>%s</i> con %s %s<br>©2010, Luis Alfonso Arce Gonzalez<br><br>Licenciado bajo <a href=\"http://www.gnu.org/licenses/gpl.html\">GNU General Public License version 3<a>.<br><br>Encuentre trucos, manuales e información sobre nuevas versiones en <a href=\"http://www.javadikt.net\">JavaDiKt.net</a></html>");
			messages.put(ABOUTWIN_borderTextLicense, "Licencias");
			messages.put(ABOUTWIN_licenseText, "<html><font size=-2>JavaDiKt no seria posible sin el esfuerzo y la generosidad de las personas que elaboraron los siguientes recursos y bibliotecas, que otorgan ademas sin coste alguno permisos de uso de sus trabajos bajo las licencias detalladas a continuación:<br><br><b><a href=\"http://www.csse.monash.edu.au/~jwb/kanjidic.html\">KANJIDIC</a></b>, por Jim Breen del <i><a href=\"http://www.edrdg.org/\">ELECTRONIC DICTIONARY RESEARCH AND DEVELOPMENT GROUP</a></i>, bajo licencia<i><a href=\"http://creativecommons.org/licenses/by-sa/3.0/\"> Creative Commons Attribution-ShareAlike3.0 Unported</a></i>.<br><br><b><a href=\"http://www.kanji.org/kanji/dictionaries/skip_permission.htm\">SYSTEM OF KANJI INDEXING BY PATTERNS(SKIP)</a></b>, por JackH alpern de la <i><a href=\"http://www.kanji.org\">Kanji Dictionary Publishing Society</a></i> bajo licencia <i><a href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\">Creative CommonsAttribution-Noncommercial-Share Alike 3.0 Unported</a></i>.<br><br><b><a href=\"http://www.neodatis.org/\">Neodatis ODB</a></b>, biblioteca bajo licencia <a href=\"http://www.gnu.org/licenses/lgpl-2.1.html\">LGPL 2.1</a>.<br><br><b><a href=\"http://www.xom.nu/\">XOM</a></b>, biblioteca bajo licencia <a href=\"http://www.gnu.org/licenses/lgpl-2.1.html\">LGPL 2.1</a>.</font></html>");

			initialized = true;
			buildKeyMap();
		
		//FIN DE IDIOMAS
		} else {
			
			initMessages(DEFAULT_LENGUAGE);
			
			localePrintln(messages.get(GEN_LENGUAJE_DOES_NOT_EXISTS_C), languaje);
		}
	}

	// TODO: javadoc
	public static String[] getList(String list) {

		String[] retrievedArray = null;
		// return sorted
		SortedSet<String> retrievedList = null;

		if (!initialized)
			initMessages();
		if (list == null)
			ExceptionHandler.handleException(new NullPointerException(), KNOWN_EXCEPTIONS.MESSAGES_LIST_NULL);

		if (listMessages.get(list) != null)
			retrievedList = new TreeSet<String>(listMessages.get(list).values());

		if (retrievedList == null)
			ExceptionHandler.handleException(new NoSuchMessage("Failed to find the message, the requested list does not exists:" + list), KNOWN_EXCEPTIONS.MESSAGES_LIST_DOES_NOT_EXISTS);

		retrievedArray = new String[retrievedList.size()];
		retrievedList.toArray(retrievedArray);

		return retrievedArray;
	}

	/**
	 * 
	 * <br>
	 * <b> initMessages</b>
	 * 
	 * <br>
	 * <br>{@code public final static void initMessages()}
	 * 
	 * <br>
	 * <br>
	 * Método que devuelve los mensajes solicitados en función de una clave de
	 * tipo String solo si antes se ha llamado al método {@code initMessages()},
	 * en caso contrario dvolverá los mensajes en el idioma por defecto. Las
	 * claves y las cadenas devueltas solo pueden modificarse en el código
	 * fuente de la clase.<br>
	 * <br>
	 * 
	 * @param key
	 *            - Cadena clave para solicitar el mensaje.
	 * @return El mensaje solicitado por la clave message.
	 * @throws NoSuchMessage
	 *             - Si el mensaje solicitado no existe.
	 */
	public final static String get(String key){
		
		Map<String,String> retrieveFrom;
		String retrievedMessage;
		
		if(!initialized)
			initMessages();
		if(key == null)
			ExceptionHandler.handleException(new NullPointerException(), KNOWN_EXCEPTIONS.MESSAGES_NULL);
		
		retrievedMessage = messages.get(key);

		if (retrievedMessage == null) {
			// TODO: warning!
			if (key.startsWith("CASE_"))
				retrieveFrom = listMessages.get(LIST_CRITERIA_CASES_INT);
			else if (key.startsWith("KANJI_"))
				retrieveFrom = listMessages.get(LIST_CRITERIA_FIELD_EXTENDED);
			else if (key.startsWith("DIC_"))
				retrieveFrom = listMessages.get(LIST_CRITERIA_VALUES_DIC);
			else if (key.startsWith("JIS_"))
				retrieveFrom = listMessages.get(LIST_CRITERIA_VALUES_JIS);
			else if (key.startsWith("READ_"))
				retrieveFrom = listMessages.get(LIST_CRITERIA_VALUES_READING_EXTENDED);
			else if (key.startsWith("LANGUAJE_"))
				retrieveFrom = listMessages.get(LIST_LANGUAGES_SUPPORTED);
			else if (key.startsWith("ORDERMETHODSINT_"))
				retrieveFrom = listMessages.get(LIST_ORDENABLE_METHODS_INTEGER);
			else if (key.startsWith("ORDERMETHODSSTR_"))
				retrieveFrom = listMessages.get(LIST_ORDENABLE_METHODS_STRING);
			else if (key.startsWith("ORDER_"))
				retrieveFrom = listMessages.get(LIST_ORDENABLE_FIELDS);
			else
				retrieveFrom = messages;

			retrievedMessage = retrieveFrom.get(key);
		}
		if (retrievedMessage == null)
			ExceptionHandler.handleException(new NoSuchMessage("Failed to find the message, the requested message does not exists:" + key), KNOWN_EXCEPTIONS.MESSAGES_MESSAGE_DOES_NOT_EXISTS);

		return retrievedMessage;
	}
	
	public static String getKey(String message) {

		return getKeys(message).get(0);
	}

	public static List<String> getKeys(String message){
		
		List<String> keyList = null;
		
		if (!initialized)
			initMessages();
		if (message == null)
			ExceptionHandler.handleException(new NullPointerException(), KNOWN_EXCEPTIONS.MESSAGES_NULL);

		keyList = keyMap.get(message);

		if (keyList == null || keyList.size() == 0)
			ExceptionHandler.handleException(new NoSuchMessage("Failed to find the key, the requested key does not exists:" + message), KNOWN_EXCEPTIONS.MESSAGES_KEY_DOES_NOT_EXISTS);
		
		return keyList;
	}

	/**
	 * 
	 * <br>
	 * <b> localePrintln</b>
	 * 
	 * <br>
	 * <br>{@code public static final void localePrintln(Object o)}
	 * 
	 * <br>
	 * <br>
	 * Método que imprime en la consola del sistema operativo local el resultado
	 * de {@code o.toString}, usando la tabla de caracteres adecuada.<br>
	 * <br>
	 * 
	 * @param o
	 *            - Objeto a imprimir por consola.
	 */
	public static final void localePrintln(Object o){

		String s = o.toString();

		if (System.getProperty("os.name").contains("Windows XP")) {

			byte[] message = new byte[s.length()];

			try {
				message = s.getBytes("IBM850");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}

			try {
				System.out.write(message);
			} catch (IOException e) {
				e.printStackTrace();
			}

			System.out.println();

		} else {

			System.out.println(s);
		}
	}

	/**
	 * 
	 * <br>
	 * <b> localePrintln</b>
	 * 
	 * <br>
	 * <br>{@code public static final void localePrintln(String s,Object...args)}
	 * 
	 * <br>
	 * <br>
	 * Método que imprime en la consola del sistema operativo local el resultado
	 * de formatear según el estandar de C una cadena, usando la tabla de
	 * caracteres adecuada.<br>
	 * <br>
	 * 
	 * @param s
	 *            - Cadena que será formateada.
	 * @param args
	 *            - Argumentos para el formateado de la cadena.
	 */
	public static final void localePrintln(String s,Object...args){
		
		localePrintln(String.format(s, args));
	}

	private static void buildKeyMap() {

		keyMap = new HashMap<String, List<String>>();
		List<String> currentList = null;

		// add simple messages keys

		for (String key : messages.keySet()) {

			currentList = keyMap.get(get(key));

			if (currentList == null) {
				currentList = new ArrayList<String>();
				keyMap.put(get(key), currentList);
			}

			currentList.add(key);
		}

		// add listed messages keys

		for (String listKey : listMessages.keySet()) {
			for (String key : listMessages.get(listKey).keySet()) {

				currentList = keyMap.get(get(key));

				if (currentList == null) {
					currentList = new ArrayList<String>();
					keyMap.put(get(key), currentList);
				}

				if (!currentList.contains(key))
					currentList.add(key);
			}
		}
	}

	public static String[] getKeyListOwners(String key) {

		List<String> owners = new ArrayList<String>();

		for (String listKey : listMessages.keySet()) {
			if (listMessages.get(listKey).containsKey(key))
				owners.add(listKey);
		}

		String[] ownersArray = new String[owners.size()];

		owners.toArray(ownersArray);

		return ownersArray;
	}

	/**
	 * Returns the key of a message in a specified list
	 * 
	 * @param message
	 * @param listKey
	 * @return
	 */
	public static String getKeyInList(String message, String listKey) {

		List<String> keys = getKeys(message);

		// TODO:improve, slow
		for (String key : keys) {
			for (String retListKey : getKeyListOwners(key)) {
				if (retListKey.equals(listKey)) {
					return key;

				}

			}
		}

		return null;
	}

	public static void main(String... Strings) {

		System.out.println(getKeys("ー"));
	}
}