//Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.core.winHandlers;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Image;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import net.ajaest.core.exporters.Exporter;
import net.ajaest.core.main.ExceptionHandler;
import net.ajaest.core.main.JDKGUIEngine;
import net.ajaest.core.main.Launcher;
import net.ajaest.core.main.Messages;
import net.ajaest.core.main.ExceptionHandler.KNOWN_EXCEPTIONS;
import net.ajaest.data.auxi.Pair;
import net.ajaest.data.auxi.SystemEnums.ISO639ー1;
import net.ajaest.data.dict.sort.KanjiOrdering;
import net.ajaest.data.dict.sort.KanjiSortExpression;
import net.ajaest.data.dict.sort.NumberOrderDef;
import net.ajaest.data.dict.sort.StringOrderDef;
import net.ajaest.data.kanji.AllowedStrokeLineEnum;
import net.ajaest.data.kanji.KanjiComparator;
import net.ajaest.data.kanji.KanjiEnums;
import net.ajaest.data.kanji.KanjiGraph;
import net.ajaest.data.kanji.KanjiStroke;
import net.ajaest.data.kanji.KanjiTag;
import net.ajaest.data.kanji.KanjiEnums.KanjiDicsEnum;
import net.ajaest.data.kanji.KanjiEnums.KanjiFieldEnum;
import net.ajaest.data.kanji.KanjiEnums.KanjiReadingTypesEnum;
import net.ajaest.gui.auxi.ExportJPanel;
import net.ajaest.gui.auxi.FixedTableModel;
import net.ajaest.gui.auxi.JLineDraw;
import net.ajaest.gui.auxi.JPanelFactory;
import net.ajaest.gui.auxi.RowColorCellRenderer;
import net.ajaest.gui.dialogs.ExportSorterAdderDialog;
import net.ajaest.gui.dialogs.FileExplorerDialog;
import net.ajaest.gui.windows.MainWindow;

//TODO: JavaDoc
public class MainWH implements WinHandler {

	// GUI engine
	private JDKGUIEngine jdkGui;
	private MainWindow mw = null;

	// Flags and state
	private Boolean showingKanjiInfo = false;
	private Boolean modifingQuery = false;
	private Boolean sortersModified = true;

	private Integer selectedTab = null;
	private static final Integer TAB_DRAW = 0;
	private static final Integer TAB_CRITERIA = 1;
	@SuppressWarnings("unused")
	private static final Integer TAB_EXPORT = 2;
	@SuppressWarnings("unused")
	private static final Integer TAB_OPTIONS = 3;

	// Win properties
	private ExportJPanel exportResultsPanel = null;
	private JScrollPane resultPanelScroll = null;

	private static final Color TABLE_PRIMARY_BACKGROUNG_COLOR = Color.WHITE;
	private static final Color TABLE_SECONDARY_BACKGROUNG_COLOR = Color.LIGHT_GRAY;

	private Map<String, KanjiGraph> kanjiGraphs;

	private FixedTableModel queryTableModel = null;
	private RowColorCellRenderer queryTableCellRenderer = null;

	private FixedTableModel exportSortingTableModel = null;

	private List<KanjiTag> exportSortedList;
	private List<KanjiFieldEnum> exportInvolvedFields;
	private List<Object> exportInvolvedValues;

	public MainWH(JDKGUIEngine jdgGui) {
		this.jdkGui = jdgGui;

		// pre window asignation init: init query tablemodel
		Object[] columnNames = new Object[4];
		columnNames[0] = getMessage(Messages.WINMAIN_QUERY_DATA_columnAndOr);
		columnNames[1] = getMessage(Messages.WINMAIN_QUERY_DATA_columnField);
		columnNames[2] = getMessage(Messages.WINMAIN_QUERY_DATA_columnThatFulfilsThat);
		columnNames[3] = getMessage(Messages.WINMAIN_QUERY_DATA_columnValue);
		int rowCount = 0;
		queryTableModel = new FixedTableModel(columnNames, rowCount);
		String orString = getMessage(Messages.WINMAIN_QUERY_DATA_cellOrString);
		queryTableCellRenderer = new RowColorCellRenderer(queryTableModel, TABLE_PRIMARY_BACKGROUNG_COLOR, TABLE_SECONDARY_BACKGROUNG_COLOR, 0, orString);
	}

	public void addAndQueryButtonPressed() {
    	
		String field = null;
		String fulfils = null;
		String value = null;
		String andOrString = null;

		field = getSelectedMessageFromFieldComboBox();
		fulfils = getSelectedMessageFromFulfilsComboBox();
		value = getSelectedMessageFromValueComboBox();
		
		if (!checkField(field, value)) {
			ExceptionHandler.handleException(new NumberFormatException(), KNOWN_EXCEPTIONS.NUMBER_CAST_EXCEPTION);
		} else {
			if (!modifingQuery) {

				Integer rowCount = queryTableModel.getRowCount();

				if (rowCount == 0) {
					mw.getCriteriaQueryMakerAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
					mw.getCriteriaQueryMakerOrButton().setEnabled(true);
					mw.getDrawAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
					mw.getDrawOrButton().setEnabled(true);

					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString);
				} else {
					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellAndString);
				}

				queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});

			} else if (modifingQuery) {

				Integer selectedRow = null;
				JTable currentTable = null;

				if (selectedTab == TAB_DRAW)
					currentTable = mw.getDrawQueryTable();
				else if (selectedTab == TAB_CRITERIA)
					currentTable = mw.getCriteriaQueryTable();

				selectedRow = currentTable.getSelectedRow();

				if (selectedRow == 0) {

					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString);

					if (selectedTab == TAB_DRAW) {
						mw.getDrawAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw.getDrawOrButton().setEnabled(true);
					} else if (selectedTab == TAB_CRITERIA) {
						mw.getCriteriaQueryMakerAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw.getCriteriaQueryMakerOrButton().setEnabled(true);
					}
				} else
					andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellAndString);;

				queryTableModel.setValueAt(andOrString, selectedRow, 0);
				queryTableModel.setValueAt(field, selectedRow, 1);
				queryTableModel.setValueAt(fulfils, selectedRow, 2);
				queryTableModel.setValueAt(value, selectedRow, 3);

				modifingQuery = false;
				setDisableForModifing(true);
			}

			mw.getCriteriaQueryTable().repaint();
			mw.getDrawQueryTable().repaint();
		}
    }

	public void changeExportPath() {
                // Maybe this functionality must be handled by exports dialogs engine
		int selectedExporter = mw.getExportFormatComboBox().getSelectedIndex();
		Exporter<KanjiTag> exporter = jdkGui.getJavaDikt().getExporters().get(selectedExporter);

		FileFilter exporterFilters = exporter.getFileChooserFilters();

		FileExplorerDialog fed = new FileExplorerDialog(exporterFilters, mw, true);
		fed.setLocale(new Locale(jdkGui.getOptions().getLanguage()));

		int retValue = fed.showOpenDialog();

		if (retValue == JFileChooser.APPROVE_OPTION) {
			File exportTo = fed.getFileChooser().getSelectedFile();
			mw.getExportPathTextField().setText(exportTo.getPath());
			mw.getExportControlsTab().updateUI();
		}

	}

	public void drawAndButtonPressed() {

		String field = null;
		String fulfils = null;
		String value = null;
		String andOrString = null;

		KanjiGraph kg;
		List<KanjiStroke> ks = new ArrayList<KanjiStroke>();
		JLineDraw draw = mw.getDrawPanel().getJLineDraw();
		if (draw != null) {
			List<AllowedStrokeLineEnum> drawnAsterisk = mw.getDrawPanel().getJLineDraw().getAsteriskModel();

			// TODO: implement stroke clues
			int order = 1;
			List<AllowedStrokeLineEnum> strokeList;
			for (Pair<Integer, Integer> stroke : draw.getStrokes()) {

				strokeList = new ArrayList<AllowedStrokeLineEnum>();

				for (int i = stroke.getFirst(); i <= stroke.getSecond(); i++) {
					strokeList.add(drawnAsterisk.get(i));
				}

				ks.add(new KanjiStroke(strokeList, order, -1));
				order++;
			}

			kg = new KanjiGraph(ks, null, -1);

			kanjiGraphs.put(kg.toString(), kg);

			field = getMessage(Messages.KANJI_GRAPH);
			// only equals is allowed
			fulfils = getMessage(Messages.CASE_EQUALS);
			value = kg.toString();

			Integer rowCount = queryTableModel.getRowCount();

			if (rowCount == 0) {
				mw.getCriteriaQueryMakerAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
				mw.getCriteriaQueryMakerOrButton().setEnabled(true);
				mw.getDrawAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
				mw.getDrawOrButton().setEnabled(true);

				andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString);
			} else {
				andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellAndString);
			}

			queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});

			mw.getCriteriaQueryTable().repaint();
			mw.getDrawQueryTable().repaint();
		}
	}

	public void drawOrButtonPressed() {

		String field = null;
		String fulfils = null;
		String value = null;
		String andOrString = null;

		KanjiGraph kg;
		List<KanjiStroke> ks = new ArrayList<KanjiStroke>();
		JLineDraw draw = mw.getDrawPanel().getJLineDraw();
		if (draw != null) {
			List<AllowedStrokeLineEnum> drawnAsterisk = mw.getDrawPanel().getJLineDraw().getAsteriskModel();

			// TODO: implement stroke clues
			int order = 1;
			List<AllowedStrokeLineEnum> strokeList;
			for (Pair<Integer, Integer> stroke : draw.getStrokes()) {

				strokeList = new ArrayList<AllowedStrokeLineEnum>();

				for (int i = stroke.getFirst(); i <= stroke.getSecond(); i++) {
					strokeList.add(drawnAsterisk.get(i));
				}

				ks.add(new KanjiStroke(strokeList, order, -1));
				order++;
			}

			kg = new KanjiGraph(ks, null, -1);

			kanjiGraphs.put(kg.toString(), kg);

			field = getMessage(Messages.KANJI_GRAPH);
			// only equals is allowed
			fulfils = getMessage(Messages.CASE_EQUALS);
			andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellOrString);
			value = kg.toString();

			queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});

			mw.getCriteriaQueryTable().repaint();
			mw.getDrawQueryTable().repaint();
		}
	}
	public void drawRemoveLastLineButtonPressed() {

		if (mw.getDrawPanel().isMouseClicked())
			mw.getDrawPanel().unclickMouse();
		else
			mw.getDrawPanel().removeLastLine();
	}

	public void drawRemoveLastStrokeButtonPressed() {

		if (mw.getDrawPanel().isMouseClicked())
			mw.getDrawPanel().unclickMouse();

		mw.getDrawPanel().removeLastStroke();
	}

    public void drawResetButtonPressed() {

		if (mw.getDrawPanel().isMouseClicked())
			mw.getDrawPanel().unclickMouse();
		mw.getDrawPanel().reset();

		mw.getCriteriaQueryTable().repaint();
		mw.getDrawQueryTable().repaint();
	}

	public void executeDrawQuery() {

		if (mw.getDrawPanel().isMouseClicked())
			mw.getDrawPanel().unclickMouse();

		setSorterListModified(true);

		executeQuery();
    }

	public void executeQuery() {

		if (queryTableModel.getRowCount() > 0) {
			// elaborates key string values. jdkgui translates it to query
			List<String> boolKeys = new ArrayList<String>();
			List<String> fieldKeys = new ArrayList<String>();
			List<String> caseKeys = new ArrayList<String>();
			List<Object> valueKeys = new ArrayList<Object>();

			List<String> tempList = null;
			Integer rowCount = queryTableModel.getRowCount();
			for (int i = 0; i < rowCount; i++) {

				// and/or column messages can have multiple keys
				tempList = Messages.getKeys((String) queryTableModel.getValueAt(i, 0));
				if (tempList.contains(Messages.WINMAIN_QUERY_DATA_cellFirstString) || tempList.contains(Messages.WINMAIN_QUERY_DATA_cellAndString))
					boolKeys.add(Messages.WINMAIN_QUERY_DATA_cellAndString);
				else
					boolKeys.add(Messages.WINMAIN_QUERY_DATA_cellOrString);

				fieldKeys.add(getSelectedKeyFromFieldComboBox((String) queryTableModel.getValueAt(i, 1)));
				caseKeys.add(Messages.getKey((String) queryTableModel.getValueAt(i, 2)));

				if (kanjiGraphs.containsKey(queryTableModel.getValueAt(i, 3))) {
					valueKeys.add(kanjiGraphs.get(queryTableModel.getValueAt(i, 3)));
				} else {
					// jdkgui decodes the value literal
					valueKeys.add(queryTableModel.getValueAt(i, 3));
				}
			}

			try {
				List<KanjiTag> currentResults = jdkGui.executeQuery(boolKeys, fieldKeys, caseKeys, valueKeys);

				mw.getResultPanel().setBorder(BorderFactory.createTitledBorder(getMessage(Messages.WINMAIN_RESULTS_BorderTextResults) + ": " + currentResults.size()));
				resultPanelScroll = JPanelFactory.createKanjiGrid(this, currentResults);
				mw.setBufferedKanjiInfoGrid(resultPanelScroll);

				JScrollPane exportScroll = JPanelFactory.createToggleKanjiGrid(this, currentResults);
				exportResultsPanel = (ExportJPanel) exportScroll.getViewport().getComponent(0);
				mw.setBufferedKanjiExportGrid(exportScroll);

				setSorterListModified(true);
				exportSortedList = null;

				setResultPanelToKanjiToInfoGrid();
			} catch (NumberFormatException e) {
				ExceptionHandler.handleException(e, KNOWN_EXCEPTIONS.NUMBER_CAST_EXCEPTION);

			}
		}
    }



    public void exportMoreButtonPressed() {
		
		jdkGui.getExporDialogsWH().exportMoreButtonPressed();
	}

	public void exportRearrangeButtonPressed() {

		List<KanjiTag> selected = null;
		
		if(exportResultsPanel != null)
			selected = exportResultsPanel.getSelectedKanji();

		if (selected != null && selected.size() > 0) {
			if (sortersModified) {

				exportSortedList = new LinkedList<KanjiTag>(selected);

				KanjiSortExpression kse = createExportSortExpression();

				if (kse != null) {
					Collections.sort(exportSortedList, new KanjiComparator(kse));
					exportSortedList = new ArrayList<KanjiTag>(exportSortedList);
				}

				if (kse != null) {
					exportInvolvedFields = kse.getInvolvedFieldsInOrdering();
					exportInvolvedValues = kse.getInvolvedValuesInOrdering();
				} else {
					exportInvolvedFields = new ArrayList<KanjiFieldEnum>();
					exportInvolvedValues = new ArrayList<Object>();
				}

				jdkGui.getExporDialogsWH().getExportRearrangerWH().setSortedList(exportSortedList, exportInvolvedFields, exportInvolvedValues);

				sortersModified = false;
			}

			jdkGui.getExporDialogsWH().invokeRearrangeDialog();
		}
    }
	
	private KanjiSortExpression createExportSortExpression(){
		
		KanjiOrdering ko = new KanjiOrdering();
		KanjiSortExpression kse = null;

		String fieldKey;
		String value = null;
		String valueKey;
		String orderByKey;
		for (int i = 0; i < exportSortingTableModel.getRowCount(); i++) {
			// _____
			fieldKey = (String) exportSortingTableModel.getValueAt(i, 2);
			String[] splitValue = fieldKey.split(" - ");
			if (splitValue.length == 2) {
				fieldKey.replace("ORDER_", "KANJI_");
				value = splitValue[1];
			}

			fieldKey = Messages.getKeyInList(splitValue[0], Messages.LIST_ORDENABLE_FIELDS);
			// _____

			orderByKey = (String) exportSortingTableModel.getValueAt(i, 1);
			switch (KanjiEnums.getFieldType(fieldKey.replace("ORDER_", "KANJI_"))) {
				case INTEGER :
					orderByKey = Messages.getKeyInList(orderByKey, Messages.LIST_ORDENABLE_METHODS_INTEGER);
					break;
				case STRING :
					orderByKey = Messages.getKeyInList(orderByKey, Messages.LIST_ORDENABLE_METHODS_STRING);
					break;
			}
			// _____

			if (fieldKey.equals(Messages.ORDER_CLASSIC_NELSON)) {
				kse = selectOrderBy(ko.sort_by_Nelson_radical(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_CLASSIC_RADICAL)) {
				kse = selectOrderBy(ko.sort_by_classic_radical(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_DE_ROO)) {
				kse = selectOrderBy(ko.sort_by_De_Roo_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_DIC_INDEX)) {
				valueKey = Messages.getKeyInList(value, Messages.LIST_CRITERIA_VALUES_DIC);
				KanjiDicsEnum kde = KanjiDicsEnum.valueOf(valueKey);
				kse = selectOrderBy(ko.sort_by_dictionary_reference().dic_enum_equals(kde), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_FOUR_CORNER)) {
				kse = selectOrderBy(ko.sort_by_four_corner_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_FREQUENCY)) {
				kse = selectOrderBy(ko.sort_by_frequency(), orderByKey);
			} else if (fieldKey.equals(Messages.ORDER_GRADE)) {
				kse = selectOrderBy(ko.sort_by_grade(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_GRAPH)) {
				kse = selectOrderBy(ko.sort_by_graph(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_JIS_CHARSET)) {
				kse = selectOrderBy(ko.sort_by_JIS_charset(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_JIS_CODE)) {
				kse = selectOrderBy(ko.sort_by_JIS_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_MEANING)) {
				ISO639ー1 lang = ISO639ー1.valueOf(jdkGui.getOptions().getLanguage().toUpperCase());
				kse = selectOrderBy(ko.sort_by_first_meaning().languaje_enum_equals(lang), orderByKey);
			} else if (fieldKey.equals(Messages.ORDER_READING)) {
				valueKey = Messages.getKeyInList(value, Messages.LIST_CRITERIA_VALUES_READING_EXTENDED);
				KanjiReadingTypesEnum kre = KanjiReadingTypesEnum.valueOf(valueKey);
				kse = selectOrderBy(ko.sort_by_first_reading().reading_enum_equals(kre), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_SKIP)) {
				kse = selectOrderBy(ko.sort_by_SKIP_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_SPAHN_HADAMITZKY)) {
				kse = selectOrderBy(ko.sort_by_SpahnーHadamitzky_code(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_STROKE_COUNT)) {
				kse = selectOrderBy(ko.sort_by_stroke_count(), orderByKey);
				ko = kse.and_if_equals();
			} else if (fieldKey.equals(Messages.ORDER_UNICODE_VALUE)) {
				kse = selectOrderBy(ko.sort_by_Unicode(), orderByKey);
				ko = kse.and_if_equals();
			}

			// _____
		}
		
		return kse;
	}

	private KanjiSortExpression selectOrderBy(StringOrderDef sod, String orderByKey) {

		KanjiSortExpression kse = null;

		if (orderByKey.equals(Messages.ORDERMETHODSSTR_ALPHABETICALLY)) {
			kse = sod.alphabetically();
		} else if (orderByKey.equals(Messages.ORDERMETHODSSTR_ALPHABETICALLY_INVERSE)) {
			kse = sod.alphabetically_inverse();
		} else if (orderByKey.equals(Messages.ORDERMETHODSINT_RAMDOM)) {
			kse = sod.ramdomly();
		} else if (orderByKey.equals(Messages.ORDERMETHODSSTR_RAMDOM)) {
			kse = sod.ramdomly();
		}

		return kse;
	}

	private KanjiSortExpression selectOrderBy(NumberOrderDef sod, String orderByKey) {

		KanjiSortExpression kse = null;

		if (orderByKey.equals(Messages.ORDERMETHODSINT_FROM_GREATEST_TO_LEAST)) {
			kse = sod.from_greatest_to_least();
		} else if (orderByKey.equals(Messages.ORDERMETHODSINT_FROM_LEAST_TO_GREATEST)) {
			kse = sod.from_least_to_greatest();
		} else if (orderByKey.equals(Messages.ORDERMETHODSINT_RAMDOM)) {
			kse = sod.ramdomly();
		} else if (orderByKey.equals(Messages.ORDERMETHODSSTR_RAMDOM)) {
			kse = sod.ramdomly();
		}

		return kse;
	}

	public void exportResultPanelInvertButtonPressed() {

		if (exportResultsPanel != null)
			exportResultsPanel.invertSelection();
	}

    public void exportResultPanelResetButtonPressed() {

		if (exportResultsPanel != null)
			exportResultsPanel.setAll(false);
	}

	public void exportResultPanelSelectAllButtonPressed() {

		if (exportResultsPanel != null)
			exportResultsPanel.setAll(true);
	}

	public ComboBoxModel getCriteriaFieldComboBoxModel() {

		String[] comboMessages;
		if (jdkGui.getOptions().getExtendedInfo()) {
			comboMessages = Messages.getList(Messages.LIST_CRITERIA_FIELD_EXTENDED);
			// TODO: remove, graph criteria yet not supported workaround
		} else
			comboMessages = Messages.getList(Messages.LIST_CRITERIA_FIELD_BASIC);
		
		return new DefaultComboBoxModel(comboMessages);
	}

	public ComboBoxModel getExportFormatModel() {

		String[] exporterNames = new String[jdkGui.getJavaDikt().getExporters().size()];

		for (int i = 0; i < jdkGui.getJavaDikt().getExporters().size(); i++) {

			exporterNames[i] = jdkGui.getJavaDikt().getExporters().get(i).getName();
		}

		ComboBoxModel cbm = new DefaultComboBoxModel(exporterNames);
		if (jdkGui.getJavaDikt().getExporters().size() > 0)
			cbm.setSelectedItem(jdkGui.getJavaDikt().getExporters().get(0).getName());

		return cbm;
	}

    public TableModel getExportSorterTableModel() {

		Object[] columnNames = new Object[3];
		columnNames[0] = getMessage(Messages.WINMAIN_EXPORT_SORTING_columnLevel);
		columnNames[1] = getMessage(Messages.WINMAIN_EXPORT_SORTING_columnOrderBy);
		columnNames[2] = getMessage(Messages.WINMAIN_EXPORT_SORTING_columnField);

		int rowCount = 0;

		exportSortingTableModel = new FixedTableModel(columnNames, rowCount);

		return exportSortingTableModel;
	}

	public ComboBoxModel getExportStyleModel() {

		String[] stylerNames = new String[jdkGui.getJavaDikt().getExporters().size()];

		ComboBoxModel cbm;
		
		if (jdkGui.getJavaDikt().getExporters().size() > 0) {

			Exporter<?> temp = jdkGui.getJavaDikt().getExporters().get(0);
			for (int i = 0; i < temp.getStylers().size(); i++) {

				stylerNames[i] = temp.getStylers().get(i).getName();
			}

			cbm = new DefaultComboBoxModel(stylerNames);
			if (temp.getStylers().size() > 0)
				cbm.setSelectedItem(temp.getStylers().get(0).getName());
		} else {
			cbm = new DefaultComboBoxModel(stylerNames);
		}

		return cbm;
	}

	public List<? extends Image> getIcons() {

		return jdkGui.getIcons();
	}

	public MainWindow getWindow() {
		return mw;
	}

    public String getMessage(String message) {

		return Messages.get(message);
	}

	public ComboBoxModel getOptionLanguageComboBoxModel() {

		return new DefaultComboBoxModel(Messages.getList(Messages.LIST_LANGUAGES_SUPPORTED));
	}

	public TableModel getQueryTableModel() {

		return queryTableModel;
	}

	public TableCellRenderer getQueryTableCellRenderer() {

		return queryTableCellRenderer;
	}

    private String getSelectedKeyFromFieldComboBox(String selectedMessage) {

		List<String> keys = Messages.getKeys(selectedMessage);
		String selectedKey = "";
		
		for (String key : keys) {
			if (key.contains("KANJI_")) {
				selectedKey = key;
				break;
			}
		}

		return selectedKey;
	}
    
	private String getSelectedMessageFromFieldComboBox() {

		int selectedItemIndex = mw.getCriteriaQueryFieldComboBox().getSelectedIndex();

		String[] currentBoxList;
		if (jdkGui.getOptions().getExtendedInfo())
			currentBoxList = Messages.getList(Messages.LIST_CRITERIA_FIELD_EXTENDED);
		else
			currentBoxList = Messages.getList(Messages.LIST_CRITERIA_FIELD_BASIC);

		return currentBoxList[selectedItemIndex];
	}

	private String getSelectedMessageFromFulfilsComboBox() {

		String fieldSelectedMessage = getSelectedMessageFromFieldComboBox();
		String fieldKey = getSelectedKeyFromFieldComboBox(fieldSelectedMessage);
		
		String[] currentBoxList = null;
		switch (KanjiEnums.getFieldType(fieldKey)) {
			case INTEGER :
				currentBoxList = Messages.getList(Messages.LIST_CRITERIA_CASES_INT);
				break;
			case STRING :
				currentBoxList = Messages.getList(Messages.LIST_CRITERIA_CASES_STRING);
				break;
		}

		return currentBoxList[mw.getCriteriaQueryFulfilsComboBox().getSelectedIndex()];
	}

	private String getSelectedMessageFromValueComboBox() {

		String valueSelectedMessage = null;

		if (mw.getCriteriaQueryValueComboBox().isEditable()) {

			valueSelectedMessage = (String) mw.getCriteriaQueryValueComboBox().getSelectedItem();
		} else {

			int selectedValueIndex = mw.getCriteriaQueryValueComboBox().getSelectedIndex();

			String selectedKey = Messages.getKey(getSelectedMessageFromFieldComboBox());

			if (selectedKey.equals(Messages.KANJI_DIC_NAME))
				valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_DIC)[selectedValueIndex];
			else if (selectedKey.endsWith(Messages.KANJI_JIS_CHARSET))
				valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_JIS)[selectedValueIndex];
			else if (selectedKey.equals(Messages.KANJI_READING_TYPE)) {
				if (jdkGui.getOptions().getExtendedInfo())
					valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_EXTENDED)[selectedValueIndex];
				else
					valueSelectedMessage = Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_BASIC)[selectedValueIndex];
			}
		}

		return valueSelectedMessage;
	}

	private void initialize() {
		selectedTab = 0;
		kanjiGraphs = new HashMap<String, KanjiGraph>();
		// pos window asignation init: query comboboxes
		mw.getCriteriaQueryFieldComboBox().setSelectedIndex(0);
		updateChoosableQueryComboBoxes();

	}

	public void invokeKanjiInfoWindow(KanjiTag kt) {
		jdkGui.getKanjiInfoWinEngine().invokeWindow(kt);
	}

	public void kanjiFileChoose() {

		FileFilter kanjiDictFileFilter = new FileFilter() {
			@Override
			public boolean accept(File f) {

				return f.isDirectory() || f.isFile() && f.getName().endsWith(".jdk");
			}
			@Override
			public String getDescription() {
				return Messages.get(Messages.FILE_DESCRIPTION_JDK_KANJI_FILE);
			}
		};
		
		FileExplorerDialog fed = new FileExplorerDialog(kanjiDictFileFilter, mw, true);
		fed.setLocale(new Locale(jdkGui.getOptions().getLanguage()));

		int retValue = fed.showOpenDialog();

		if (retValue == JFileChooser.APPROVE_OPTION) {
			File kanjiDict = fed.getFileChooser().getSelectedFile();
			mw.getOptionKanjiDictPathTextField().setText(kanjiDict.getAbsolutePath());
			mw.getOptionControlsTab().updateUI();
		}
    }

    public void orQueryButtonPressed() {

		String field = null;
		String fulfils = null;
		String value = null;
		String andOrString = null;

		andOrString = getMessage(Messages.WINMAIN_QUERY_DATA_cellOrString);
		field = getSelectedMessageFromFieldComboBox();
		fulfils = getSelectedMessageFromFulfilsComboBox();
		value = getSelectedMessageFromValueComboBox();

		if (!checkField(field, value)) {
			if (!field.equals(KanjiFieldEnum.KANJI_GRAPH))
				ExceptionHandler.handleException(new NumberFormatException(), KNOWN_EXCEPTIONS.NUMBER_CAST_EXCEPTION);
		} else {
			if (!modifingQuery) {
				queryTableModel.addRow(new Object[]{andOrString, field, fulfils, value});
			} else if (modifingQuery) {

				Integer selectedRow = null;
				JTable currentTable = null;

				if (selectedTab == TAB_DRAW)
					currentTable = mw.getDrawQueryTable();
				else if (selectedTab == TAB_CRITERIA)
					currentTable = mw.getCriteriaQueryTable();

				selectedRow = currentTable.getSelectedRow();

				queryTableModel.setValueAt(andOrString, selectedRow, 0);
				queryTableModel.setValueAt(field, selectedRow, 1);
				queryTableModel.setValueAt(fulfils, selectedRow, 2);
				queryTableModel.setValueAt(value, selectedRow, 3);

				modifingQuery = false;
				setDisableForModifing(true);
			}

			mw.getCriteriaQueryTable().repaint();
			mw.getDrawQueryTable().repaint();
		}
    }

	public void QueryModifierModifyButtonPressed() {

		Integer selectedRow = null;
		JTable currentTable = null;

		if (selectedTab == TAB_DRAW)
			currentTable = mw.getDrawQueryTable();
		else if (selectedTab == TAB_CRITERIA)
			currentTable = mw.getCriteriaQueryTable();


		selectedRow = currentTable.getSelectedRow();

		// Changes the selection in order to mark only the row that is being
		// modified
		currentTable.getSelectionModel().setSelectionInterval(selectedRow, selectedRow);
		currentTable.updateUI();
		// TODO: you can do much better!
		if (selectedRow != -1) {

			if (selectedRow == 0) {
				if (selectedTab == TAB_DRAW) {
					// No "and" or "or" for first column is allowed
					mw.getDrawAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerChangeText));
					mw.getDrawOrButton().setEnabled(false);

				} else if (selectedTab == TAB_CRITERIA) {
					// No "and" or "or" for first column is allowed
					mw.getCriteriaQueryMakerAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerChangeText));
					mw.getCriteriaQueryMakerOrButton().setEnabled(false);
				}
			}

			if (modifingQuery) {
				setDisableForModifing(true);
				// restores and/or buttons
				if (selectedRow == 0) {
					if (selectedTab == TAB_DRAW) {
						// No "and" or "or" for first column is allowed
						mw.getDrawAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw.getDrawOrButton().setEnabled(true);

					} else if (selectedTab == TAB_CRITERIA) {
						// No "and" or "or" for first column is allowed
						mw.getCriteriaQueryMakerAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAndText));
						mw.getCriteriaQueryMakerOrButton().setEnabled(true);
					}
				}
			} else 
				setDisableForModifing(false);
			
			modifingQuery = !modifingQuery;
		}

		mw.getCriteriaQueryTable().repaint();
		mw.getDrawQueryTable().repaint();
	}

	public void QueryModifierRemoveButtonPressed() {
		
		int[] selectedRow = null;
		JTable currentTable = null;

		if (selectedTab == TAB_DRAW)
			currentTable = mw.getDrawQueryTable();
		else if (selectedTab == TAB_CRITERIA)
			currentTable = mw.getCriteriaQueryTable();

		selectedRow = currentTable.getSelectedRows();

		int lastRemovedRow = 0;
		
		for(int i=0; i<selectedRow.length; i++){
			// -i because when a row is deleted the next rows change their
			// position
			queryTableModel.removeRow(selectedRow[i] - i);
			// We cannot expected that the rows are sorted
			if (selectedRow[i] > lastRemovedRow)
				lastRemovedRow = selectedRow[i]-i;
		}
		
		if (queryTableModel.getRowCount() > lastRemovedRow)
			currentTable.changeSelection(lastRemovedRow, 1, false, false);
		else
			currentTable.changeSelection(currentTable.getRowCount() - 1, 1, false, false);

		if (queryTableModel.getRowCount() == 0) {
			mw.getCriteriaQueryMakerAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
			mw.getCriteriaQueryMakerOrButton().setEnabled(false);
			mw.getDrawAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
			mw.getDrawOrButton().setEnabled(false);
		} else
			// if the first row is deleted, the new first row and/or attribute
			// must be "-"
			queryTableModel.setValueAt(getMessage(Messages.WINMAIN_QUERY_DATA_cellFirstString), 0, 0);

		mw.getCriteriaQueryTable().repaint();
		mw.getDrawQueryTable().repaint();
	}

	public void QueryModifierResetButtonPressed() {

		int size = queryTableModel.getRowCount();

		for (int i = 0; i < size; i++) {
			queryTableModel.removeRow(queryTableModel.getRowCount() - 1);

		}

		mw.getCriteriaQueryMakerAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
		mw.getCriteriaQueryMakerOrButton().setEnabled(false);
		mw.getDrawAndButton().setText(getMessage(Messages.WINMAIN_CRITERIA_ButtonQueryMakerAddText));
		mw.getDrawOrButton().setEnabled(false);
	}

	public void saveConfig() {

		// TODO: not restart needed to apply changes

		String lang = Messages.getKey((String) mw.getOptionLanguajeComboBox().getSelectedItem());
		lang = lang.substring("LANGUAJE_".length());
    	String kanjipath = (String) mw.getOptionKanjiDictPathTextField().getText();
		String strokepath = (String) mw.getOptionStrokesPathTextField().getText();
    	Boolean extInfo = mw.getOptionExtendedInfoCheckBox().isSelected();
    	Boolean extKanji = mw.getOptionExtendedIKanjiListCheckBox().isSelected();
    	Boolean romaji = mw.getOptionKanaToRomajiCheckBox().isSelected();

		jdkGui.getJavaDikt().saveOptions(lang, kanjipath, strokepath, extInfo, extKanji, romaji);

		jdkGui.reinitEngine();
    }

	public void setDisableForModifing(Boolean disable) {
		
		mw.getControlsTabbedPane().setEnabled(disable);

		if (selectedTab == TAB_CRITERIA) {
			mw.getCriteriaQueryModifierRemoveButton().setEnabled(disable);
			mw.getCriteriaQueryModifierResetButton().setEnabled(disable);
			mw.getCriteriaQueryControlsExecuteButton().setEnabled(disable);
			mw.getCriteriaQueryTable().setEnabled(disable);
			// I don't know why renderer dissapears when disabled
			mw.getCriteriaQueryTable().setDefaultRenderer(Object.class, getQueryTableCellRenderer());

		} else if (selectedTab == TAB_DRAW) {
			mw.getDrawQueryModifierRemoveButton().setEnabled(disable);
			mw.getDrawQueryModifierResetButton().setEnabled(disable);
			mw.getDrawExecuteButton().setEnabled(disable);
			mw.getDrawQueryTable().setEnabled(disable);
			// I don't know why renderer dissapears when disabled
			mw.getDrawQueryTable().setDefaultRenderer(Object.class, getQueryTableCellRenderer());
		}

		mw.getCriteriaQueryTable().repaint();
		mw.getDrawQueryTable().repaint();
	}

	private void setResultPanelToKanjiExportGrid() {

		JPanel results = mw.getResultPanel();

		// remove void panel if first or previous panel
		results.remove(0);

		showingKanjiInfo = false;

		results.add(mw.getExportPanel(), BorderLayout.CENTER);

		results.updateUI();
	}

	private void setResultPanelToKanjiToInfoGrid() {

		JPanel results = mw.getResultPanel();

		// remove void panel if first or previous panel
		results.remove(0);

		showingKanjiInfo = true;

		results.add(mw.getBufferedKanjiInfoGrid(), BorderLayout.CENTER);

		results.updateUI();
	}

	public void setWindow(MainWindow mw) {
		this.mw = mw;
		initialize();

	}

	public void strokeFileChoose() {
        FileFilter kanjiDictFileFilter = new FileFilter() {
			@Override
			public boolean accept(File f) {

				return f.isDirectory() || f.isFile() && f.getName().endsWith(".txt");
			}
			@Override
			public String getDescription() {
				return Messages.get(Messages.FILE_DESCRIPTION_TXT_STROKE_FILE);
			}
		};

		FileExplorerDialog fed = new FileExplorerDialog(kanjiDictFileFilter, mw, true);
		fed.setLocale(new Locale(jdkGui.getOptions().getLanguage()));

		int retValue = fed.showOpenDialog();

		if (retValue == JFileChooser.APPROVE_OPTION) {
			File kanjiDict = fed.getFileChooser().getSelectedFile();
			mw.getOptionKanjiDictPathTextField().setText(kanjiDict.getAbsolutePath());
			mw.getOptionControlsTab().updateUI();
		}
    }

    public void tabChangedActionPerformed(int index) {
		
    	selectedTab = index;

		if (mw.getBufferedKanjiInfoGrid() != null) {

			switch (selectedTab) {
				// stroke and criteria tab
				case 0 :
				case 1 :
					// if getBufferedKanjiInfoGrid() exists but another panel is
					// being displayed in results panel
					if (!showingKanjiInfo)
						setResultPanelToKanjiToInfoGrid();

					break;
				// export tab
				case 2 :
					setResultPanelToKanjiExportGrid();
					break;
				// options tab
				case 3 :
					updateOptionPanel();
					break;
			}

		} else {
			switch (selectedTab) {
				case 3 :
					updateOptionPanel();
					break;
			}

		}
    }

	public void updateChoosableQueryComboBoxes() {
    	
		String selectedMessage = getSelectedMessageFromFieldComboBox();
		String selectedKey = getSelectedKeyFromFieldComboBox(selectedMessage);

		ComboBoxModel fullfilModel = null;
		ComboBoxModel valueModel = null;

		System.out.println(selectedKey + ", " + KanjiEnums.getFieldType(selectedKey));
		switch (KanjiEnums.getFieldType(selectedKey)) {
			case INTEGER:
				fullfilModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_CASES_INT));
				break;
			case STRING:
				fullfilModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_CASES_STRING));
				break;
			case INTEGER_LIST:
				fullfilModel = new DefaultComboBoxModel();
				break;

		}

		if (selectedKey.equals(Messages.KANJI_DIC_NAME)) {

			valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_DIC));
			mw.getCriteriaQueryValueComboBox().setEditable(false);

		} else if (selectedKey.endsWith(Messages.KANJI_JIS_CHARSET)) {

			valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_JIS));
			mw.getCriteriaQueryValueComboBox().setEditable(false);

		} else if (selectedKey.equals(Messages.KANJI_READING_TYPE)) {

			if (jdkGui.getOptions().getExtendedInfo())
				valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_EXTENDED));
			else
				valueModel = new DefaultComboBoxModel(Messages.getList(Messages.LIST_CRITERIA_VALUES_READING_BASIC));

			mw.getCriteriaQueryValueComboBox().setEditable(false);

		} else {

			valueModel = new DefaultComboBoxModel();
			mw.getCriteriaQueryValueComboBox().setEditable(true);
		}

		mw.getCriteriaQueryFulfilsComboBox().setModel(fullfilModel);
		mw.getCriteriaQueryFulfilsComboBox().setSelectedIndex(0);
		mw.getCriteriaQueryFulfilsComboBox().setEnabled(true);

		mw.getCriteriaQueryValueComboBox().setModel(valueModel);
		if (!mw.getCriteriaQueryValueComboBox().isEditable())
			mw.getCriteriaQueryFulfilsComboBox().setSelectedIndex(0);
		mw.getCriteriaQueryValueComboBox().setEnabled(true);

    }

	public void updateOptionPanel() {
    	
		mw.getOptionLanguajeComboBox().setSelectedItem(jdkGui.getOptions().getLanguage());
		mw.getOptionKanjiDictPathTextField().setText(jdkGui.getOptions().getKanjiFile().getAbsolutePath());
		mw.getOptionStrokesPathTextField().setText(jdkGui.getOptions().getStrokeFile().getAbsolutePath());
		mw.getOptionExtendedInfoCheckBox().setSelected(jdkGui.getOptions().getExtendedInfo());
		mw.getOptionExtendedIKanjiListCheckBox().setSelected(jdkGui.getOptions().getExtendedIKanjiList());
		mw.getOptionKanaToRomajiCheckBox().setSelected(jdkGui.getOptions().getRomanized());

		mw.getOptionControlsTab().updateUI();
    }

	public static void main(String... args) {

		Launcher.main();
	}

	public void exportSorterAddButtonPressed() {
		jdkGui.getExporDialogsWH().invokeAddSorterDialog();

	}

	public void exportSorterResetButtonPressed() {
		mw.getExportSortingTable().setModel(getExportSorterTableModel());

	}

	public void exportSorterRemoveButtonPressed() {

		int[] selectedRow = mw.getExportSortingTable().getSelectedRows();

		int lastRemovedRow = 0;

		for (int i = 0; i < selectedRow.length; i++) {
			// -i because when a row is deleted the next rows change their
			// position
			exportSortingTableModel.removeRow(selectedRow[i] - i);
			// We cannot expected that the rows are sorted
			if (selectedRow[i] > lastRemovedRow)
				lastRemovedRow = selectedRow[i] - i;
		}

		if (exportSortingTableModel.getRowCount() > lastRemovedRow)
			mw.getExportSortingTable().changeSelection(lastRemovedRow, 1, false, false);
		else
			mw.getExportSortingTable().changeSelection(exportSortingTableModel.getRowCount() - 1, 1, false, false);

		// level literals must be re-written

		for (int i = 0; i < exportSortingTableModel.getRowCount(); i++) {
			exportSortingTableModel.setValueAt(new Integer(i + 1).toString(), i, 0);
		}

		// if add sorter is visible level combo box must be modified
		
		ExportSorterAdderDialog esaD = jdkGui.getExporDialogsWH().getExportSorterAdderWH().getExportSorterAdderDialog();
		
		if (esaD.isVisible()) {
			esaD.getLevelComboBox().setModel(jdkGui.getExporDialogsWH().getExportSorterAdderWH().getLevelComboBoxModel());
		}

		esaD.getLevelComboBox().setSelectedIndex(exportSortingTableModel.getRowCount());

		mw.getExportSortingTable().repaint();
	}

	public void setSorterListModified(boolean b) {
		this.exportSortedList = null;
		this.sortersModified = b;
	}

	public void exportExportButtonPressed() {

		String path = mw.getExportPathTextField().getText();

		if (path != null && !path.equals("")) {

			Exporter<KanjiTag> exporter;

			int exporterIdx = mw.getExportFormatComboBox().getSelectedIndex();
			exporter = jdkGui.getJavaDikt().getExporters().get(exporterIdx);
			int stylerIdx = mw.getExportStyleComboBox().getSelectedIndex();
			exporter.setSelectedStyler(stylerIdx);

			File f = new File(path);
			
			exporter.setExportFile(f);

			if (exportSortedList == null)
				exportSortedList = exportResultsPanel.getSelectedKanji();

			exporter.export(exportSortedList);

			if (exporter.getExportException() == null) {
				jdkGui.getInfoDialgosEngine().invokeInfoDialog(getMessage(Messages.INFODIALOG_title_info), getMessage(Messages.INFODIALOG_infoMessage_exportSuccess));
			} else {
				ExceptionHandler.handleException(exporter.getExportException(), KNOWN_EXCEPTIONS.EXPORT_IO);
			}
		}

	}

	public void setExportSortedList(List<KanjiTag> sortedList) {

		this.exportSortedList = sortedList;
	}

	private boolean checkField(String message, String value) {

		boolean passCheck = true;
		
		String key = Messages.getKeyInList(message, Messages.LIST_CRITERIA_FIELD_EXTENDED);

		switch (KanjiEnums.getFieldType(key)) {
			case INTEGER :
			case INTEGER_LIST :
				try{
					new Integer(value);
				} catch (NumberFormatException e) {
					passCheck = false;
				}
				break;
			default :
				passCheck = true;
				break;
		}

		return passCheck;
	}

	public void optionsAboutButtonPressed() {
		jdkGui.getAboutWH().invokeAboutWindow();
	}

	public Font getUnicodeFont() {

		return jdkGui.getUnicodeFont();
	}
}
