//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.data.auxi;

import java.util.HashSet;
import java.util.Set;

/**
 * Generic class that stores a list of elements associated to a key.
 * 
 * @author Luis Alfonso Arce González
 * 
 * @param <Key>
 *            Key common to all stored elements.
 * @param <E>
 *            Elements to be stored.
 */
public class ListEntry<Key, E> {

	private Key key;
	private Set<E> elements;

	public ListEntry(Key key) {
		super();
		this.key = key;
		elements = new HashSet<E>();
	}

	/**
	 * Returns the key. Read the constructor or class info for further
	 * information about the meaning of "key".
	 */
	public Key getKey() {
		return key;
	}

	/**
	 * Sets the key overriding the previous one. Read the constructor or class
	 * info for further information about the meaning of "key".
	 * 
	 * @param key
	 */
	public void setKey(Key key) {
		this.key = key;
	}

	/**
	 * Returns the list of elements. Read the constructor or class info for
	 * further information about the meaning of "elements".
	 */
	public Set<E> getElements() {
		return elements;
	}

	/**
	 * Sets the list pf elements overriding the previous one. Read the
	 * constructor or class info for further information about the meaning of
	 * "elements".
	 * 
	 * @param elements
	 *            {@code List} of elements
	 */
	public void setElements(Set<E> elements) {
		this.elements = elements;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((elements == null) ? 0 : elements.hashCode());
		result = prime * result + ((key == null) ? 0 : key.hashCode());
		return result;
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ListEntry other = (ListEntry) obj;
		if (elements == null) {
			if (other.elements != null)
				return false;
		} else if (!elements.equals(other.elements))
			return false;
		if (key == null) {
			if (other.key != null)
				return false;
		} else if (!key.equals(other.key))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "ListEntry [elements=" + elements + ", key=" + key + "]";
	}

}
