//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.data.dict.query;

import java.util.Set;

import net.ajaest.data.auxi.SystemEnums.ISO639ー1;
import net.ajaest.data.auxi.database.ConnectorEnum;
import net.ajaest.data.auxi.database.QAbout;
import net.ajaest.data.auxi.database.QueryCaseEnum;
import net.ajaest.data.dict.query.kanjiFields.ClassicNelsonQAbout;
import net.ajaest.data.dict.query.kanjiFields.ClassicRadicalQAbout;
import net.ajaest.data.dict.query.kanjiFields.CommonStrokeMiscountQAbout;
import net.ajaest.data.dict.query.kanjiFields.DeRooQAbout;
import net.ajaest.data.dict.query.kanjiFields.DicNameQAbout;
import net.ajaest.data.dict.query.kanjiFields.DicReferenceQAbout;
import net.ajaest.data.dict.query.kanjiFields.FourCornerQAbout;
import net.ajaest.data.dict.query.kanjiFields.FrequencyQAbout;
import net.ajaest.data.dict.query.kanjiFields.GradeQAbout;
import net.ajaest.data.dict.query.kanjiFields.JISCharsetQAbout;
import net.ajaest.data.dict.query.kanjiFields.JISCodeQAbout;
import net.ajaest.data.dict.query.kanjiFields.JLPTLevelQAbout;
import net.ajaest.data.dict.query.kanjiFields.KanjiGraphQAbout;
import net.ajaest.data.dict.query.kanjiFields.MeaningLanguageQAbout;
import net.ajaest.data.dict.query.kanjiFields.MeaningQAbout;
import net.ajaest.data.dict.query.kanjiFields.ReadingQAbout;
import net.ajaest.data.dict.query.kanjiFields.SkipQAbout;
import net.ajaest.data.dict.query.kanjiFields.SpahnHadamitzkyQAbout;
import net.ajaest.data.dict.query.kanjiFields.StrokeCountQAbout;
import net.ajaest.data.dict.query.kanjiFields.UnicodeValueQAbout;
import net.ajaest.data.dict.query.kanjiFields.VariantReferenceQAbout;
import net.ajaest.data.dict.query.kanjiFields.VariantTypeQAbout;
import net.ajaest.data.kanji.KanjiGraph;
import net.ajaest.data.kanji.SimilarKanjiStrokeDemuxer;

//TODO: JavaDoc
public class QueryAdaptor {

	private static boolean DEMUX = false;
	private static boolean SELECT_MEANING_LANGUAJE = false;
	private static boolean LIMIT_TO_JÔYÔ_KANJI = false;

	private static Integer DEMUXES;
	private static SimilarKanjiStrokeDemuxer GRAPHDEMUXER;

	private static ISO639ー1 MEANINGLANG;

	private boolean limitToJöyöKanji;
	private boolean demux;
	private boolean selectMeaningLanguaje;

	private Integer demuxes;
	private SimilarKanjiStrokeDemuxer graphDemuxer;

	private ISO639ー1 meaningLang;

	private QueryAdaptor(SimilarKanjiStrokeDemuxer graphDemuxer, boolean demux, Integer demuxes, boolean selectMeaningLanguaje, ISO639ー1 meaningLang, boolean limitToJöyöKanji) {

		this.demux = demux;
		this.selectMeaningLanguaje = selectMeaningLanguaje;
		this.limitToJöyöKanji = limitToJöyöKanji;

		this.graphDemuxer = graphDemuxer;
		this.demuxes = demuxes;

		this.meaningLang = meaningLang;

		DEMUX = demux;
		SELECT_MEANING_LANGUAJE = selectMeaningLanguaje;
		LIMIT_TO_JÔYÔ_KANJI = limitToJöyöKanji;

		GRAPHDEMUXER = graphDemuxer;
		DEMUXES = demuxes;

		MEANINGLANG = meaningLang;
	}

	public static QueryAdaptor demuxGraphs(SimilarKanjiStrokeDemuxer sksd, Integer demuxes) {

		return new QueryAdaptor(sksd, true, demuxes, SELECT_MEANING_LANGUAJE, MEANINGLANG, LIMIT_TO_JÔYÔ_KANJI);
	}

	public static QueryAdaptor meaningsInLanguaje(ISO639ー1 lang) {

		return new QueryAdaptor(GRAPHDEMUXER, DEMUX, DEMUXES, true, lang, LIMIT_TO_JÔYÔ_KANJI);
	}

	public static QueryAdaptor limitToJôyô() {

		return new QueryAdaptor(GRAPHDEMUXER, DEMUX, DEMUXES, SELECT_MEANING_LANGUAJE, MEANINGLANG, true);
	}

	public KanjiExpression adapt(KanjiExpression ke) {

		KanjiExpression adaptedKe = null;
		KanjiQuery kq;

		KanjiExpression subKe;
		QAbout domain;
		for (int i = 0; i < ke.getDomains().size(); i++) {
			domain = ke.getDomains().get(i);

			kq = selectConnector(adaptedKe, domain.getBooleanCases().get(i));

			if (domain instanceof KanjiExpression) {
				subKe = adapt((KanjiExpression) domain);
				adaptedKe = addDomain(kq, subKe);
			} else if (demux && (domain instanceof KanjiGraphQAbout)) {
				subKe = demuxGraph((KanjiGraphQAbout) domain);
				if (subKe != null)
					adaptedKe = addDomain(kq, subKe);
				else
					adaptedKe = addDomain(kq, domain);
			} else if (selectMeaningLanguaje && (domain instanceof MeaningQAbout)) {
				kq = kq.meaning_language().equal(meaningLang.toString().toLowerCase()).AND();
				adaptedKe = addDomain(kq, domain);
			} else if (selectMeaningLanguaje && (domain instanceof MeaningLanguageQAbout)) {
				adaptedKe = selectCase(kq.meaning_language(), ((MeaningLanguageQAbout) domain).getQueryCase(), meaningLang.toString().toLowerCase());
			} else {
				adaptedKe = addDomain(kq, domain);
			}
		}

		if (limitToJöyöKanji) {
			adaptedKe = new KanjiQuery().expression(adaptedKe).AND().grade().equalsOrLessThan(10);
		}

		return adaptedKe;
	}

	private KanjiExpression demuxGraph(KanjiGraphQAbout kgqa) {

		Set<KanjiGraph> demuxed = graphDemuxer.getSimilar(kgqa.getValue(), demuxes);
		
		KanjiExpression subKe = null;
		KanjiQuery kq = new KanjiQuery();
		for(KanjiGraph kg : demuxed){
			subKe = kq.kanji_graph().equal(kg);
			kq = subKe.OR();
		}

		return subKe;
	}
	
	private KanjiQuery selectConnector(KanjiExpression ke, ConnectorEnum connector) {

		switch (connector) {
			case AND :
				return ke.AND();
			case OR :
				return ke.OR();
			case FIRST :
				return new KanjiQuery();
			default :
				throw new IllegalArgumentException();
		}
	}

	private KanjiExpression addDomain(KanjiQuery kq, QAbout domain) {

		KanjiExpression ke;

		if (domain instanceof KanjiExpression) {
			ke = kq.expression((KanjiExpression) domain);
		} else if (domain instanceof StringValueQAbout) {

			StringValueQAbout strDomain = (StringValueQAbout) domain;

			if (domain instanceof DeRooQAbout) {
				ke = selectCase(kq.De_Roo_code(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof DicNameQAbout) {
				ke = selectCase(kq.dictionary_name(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof DicReferenceQAbout) {
				ke = selectCase(kq.dictionary_reference(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof FourCornerQAbout) {
				ke = selectCase(kq.four_corner_code(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof JISCharsetQAbout) {
				ke = selectCase(kq.JIS_charset(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof JISCodeQAbout) {
				ke = selectCase(kq.JIS_code_value(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof MeaningLanguageQAbout) {
				ke = selectCase(kq.meaning_language(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof MeaningQAbout) {
				ke = selectCase(kq.meaning(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof ReadingQAbout) {
				ke = selectCase(kq.reading(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof SkipQAbout) {
				ke = selectCase(kq.skip_code(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof SpahnHadamitzkyQAbout) {
				ke = selectCase(kq.SpahnーHadamizky_code(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof VariantReferenceQAbout) {
				ke = selectCase(kq.variant_reference(), strDomain.getQueryCase(), strDomain.getValue());
			} else if (domain instanceof VariantTypeQAbout) {
				ke = selectCase(kq.variant_type(), strDomain.getQueryCase(), strDomain.getValue());
			} else {
				throw new IllegalArgumentException();
			}


		} else if (domain instanceof IntegerValueQAbout) {

			IntegerValueQAbout intDomain = (IntegerValueQAbout) domain;

			if (domain instanceof ClassicNelsonQAbout) {
				ke = selectCase(kq.classic_nelson(), intDomain.getQueryCase(), intDomain.getValue());
			} else if (domain instanceof ClassicRadicalQAbout) {
				ke = selectCase(kq.classic_radical(), intDomain.getQueryCase(), intDomain.getValue());
			} else if (domain instanceof CommonStrokeMiscountQAbout) {
				ke = selectCase(kq.stroke_miscount(), intDomain.getQueryCase(), intDomain.getValue());
			} else if (domain instanceof FrequencyQAbout) {
				ke = selectCase(kq.frequency(), intDomain.getQueryCase(), intDomain.getValue());
			} else if (domain instanceof GradeQAbout) {
				ke = selectCase(kq.grade(), intDomain.getQueryCase(), intDomain.getValue());
			} else if (domain instanceof JLPTLevelQAbout) {
				ke = selectCase(kq.JLPT_level(), intDomain.getQueryCase(), intDomain.getValue());
			} else if (domain instanceof StrokeCountQAbout) {
				ke = selectCase(kq.stroke_count(), intDomain.getQueryCase(), intDomain.getValue());
			} else if (domain instanceof UnicodeValueQAbout) {
				ke = selectCase(kq.unicode_value(), intDomain.getQueryCase(), intDomain.getValue());
			} else {
				throw new IllegalArgumentException();
			}

		} else {
			if (domain instanceof ValueQAbout<?>) {
				
				Object unknowValue = ((ValueQAbout<?>) domain).getValue();
				
				if(unknowValue instanceof KanjiGraph){
					ke = selectCase(kq.kanji_graph(), ((ValueQAbout<?>) domain).getQueryCase(), (KanjiGraph) unknowValue);
				} else {
					throw new IllegalArgumentException();
				}
			} else {
				throw new IllegalArgumentException();
			}

		}

		return ke;
	}

	private <E> KanjiExpression selectCase(ValueQAbout<E> vqa, QueryCaseEnum caze, E value) {

		switch (caze) {
			case EQUALS:
				return vqa.equal(value);
			case GREATER_OR_EQUALS_THAN:
				return vqa.equaslOrGreatherThan(value);
			case GREATER_THAN:
				return vqa.greatherThan(value);
			case LESS_OR_EQUALS_THAN:
				return vqa.equalsOrLessThan(value);
			case LESS_THAN:
				return vqa.lessThan(value);
			case NOT_EQUALS :
				return vqa.notEquals(value);
			case NULL :
				return vqa.isNull();
			default :
				throw new IllegalArgumentException();
				
		}
	}
}
