//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.data.kanji;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Class that represents the graph info of a kanji.
 * 
 * @author Luis Alfonso Arce González
 * 
 */
public class KanjiGraph implements KanjiReference, Comparable<KanjiGraph>, Serializable {

	private static final long serialVersionUID = 3496003178888546250L;
	// WARNING: changing field names could cause neodatis database problems.
	private List<KanjiStroke> strokes;
	private Set<KanjiStrokeClue> strokesClues;
	private Integer unicodeRef;

	public KanjiGraph(List<KanjiStroke> strokes, Set<KanjiStrokeClue> strokesClues, Integer unicodeRef) {
		super();
		if (unicodeRef == null)
			throw new IllegalArgumentException("unicode reference is null");
		if (strokes == null)
			throw new IllegalArgumentException("stroke list is null");

		this.unicodeRef = unicodeRef;
		this.strokes = strokes;
		if (strokesClues == null) {
			this.strokesClues = new HashSet<KanjiStrokeClue>();
		} else {
			this.strokesClues = strokesClues;

		}
	}

	// TODO: add to diagram
	public Integer strokeCount() {
		return strokes.size();
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((strokes == null) ? 0 : strokes.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		KanjiGraph other = (KanjiGraph) obj;
		if (strokes == null) {
			if (other.strokes != null)
				return false;
		} else if (!strokes.equals(other.strokes))
			return false;
		return true;
	}

	/**
	 * 
	 * @return The sorted list of strokes of the kanji
	 */
	public List<KanjiStroke> getStrokes() {
		return strokes;
	}

	/**
	 * 
	 * @return The list of spacial relacions between the strokes
	 */
	public Set<KanjiStrokeClue> getStrokesClues() {
		return strokesClues;
	}

	@Override
	public Integer getUnicodeRef() {
		return unicodeRef;
	}

	@Override
	public String toString() {

		StringBuilder sb = new StringBuilder();

		for (KanjiStroke st : strokes)
			sb.append(st.toString());

		for (KanjiStrokeClue ksc : strokesClues) {
			sb.append(", ");
			sb.append(ksc.toString());
		}

		return sb.toString();

	}

	@Override
	public int compareTo(KanjiGraph o) {

		int compare = this.strokeCount().compareTo(o.strokeCount());

		if (compare == 0) {
			for (int i = 0; i < strokeCount(); i++) {
				compare = this.getStrokes().get(i).compareTo(o.getStrokes().get(i));
				if (compare != 0)
					break;
			}
		}

		return compare;
	}

	public List<AllowedStrokeLineEnum> getLineSequence() {

		List<AllowedStrokeLineEnum> sequence = new ArrayList<AllowedStrokeLineEnum>();

		for (KanjiStroke ks : getStrokes()) {
			for (AllowedStrokeLineEnum asl : ks.getStrokeList()) {

				sequence.add(asl);
			}
		}

		return sequence;
	}

}
