//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.gui.auxi;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

//TODO: JavaDoc
public class MoveRowTableModel extends AbstractTableModel implements TableModel {

	// TODO: not tested deeply, only jdk related features were tested

	private static final long serialVersionUID = 6087418158731241386L;

	protected List<List<Object>> rows;
	protected List<String> columnNames;
	protected boolean cellEditable = true;
	
	public MoveRowTableModel(List<String> columnNames) {

		if (columnNames == null)
			throw new NullPointerException();

		rows = new LinkedList<List<Object>>();
		this.columnNames = columnNames;
	}

	public List<?> getColumn(int columnIndex) {

		List<Object> column = new ArrayList<Object>();

		for (List<Object> row : rows)
			column.add(row.get(columnIndex));

		return column;
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {

		Class<?> superClass;

		if (rows.size() > 0) {
			
			List<Class<?>> superClasses;
			superClasses = getSuperClasses(rows.get(0).get(columnIndex).getClass());

			for (int rowIndex = 1; rowIndex < rows.size(); rowIndex++) {
				List<Class<?>> temp = getSuperClasses(rows.get(rowIndex).get(columnIndex).getClass());
				superClasses.retainAll(temp);
			}

			superClass = superClasses.get(superClasses.size() - 1);

		} else {
			superClass = Object.class;
		}

		return superClass;
	}

	@Override
	public int getColumnCount() {

		return columnNames.size();
	}

	@Override
	public String getColumnName(int columnIndex) {

		return columnNames.get(columnIndex);
	}

	@Override
	public int getRowCount() {

		return rows.size();
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {

		return rows.get(rowIndex).get(columnIndex);
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {

		return cellEditable;
	}


	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {

		this.rows.get(rowIndex).set(columnIndex, rowIndex);
		fireTableCellUpdated(rowIndex, columnIndex);
	}

	public void setCellEditable(boolean b) {

		cellEditable = b;
	}

	public void addRow(List<Object> row) {

		for (int i = row.size(); i < columnNames.size(); i++) {
			row.add(new Object());
		}

		rows.add(row);

		fireTableRowsInserted(rows.size() - 1, rows.size() - 1);
	}

	public List<Object> removeRow(int rowIndex) {

		List<Object> removed = rows.remove(rowIndex);

		fireTableRowsDeleted(rowIndex, rowIndex);

		return removed;
	}
	
	public int[] moveRowsTo(int rowIndex, int[] selRows) {

		int[] newPositions;

		if (selRows.length > 0 && rowIndex <= rows.size()) {

			int[] workRows = Arrays.copyOf(selRows, selRows.length);
			Arrays.sort(workRows);

			List<Integer> bgIntervalIdx = new ArrayList<Integer>();
			List<Integer> edIntervalIdx = new ArrayList<Integer>();
			bgIntervalIdx.add(workRows[0]);
			edIntervalIdx.add(workRows[0]);

			int offsetRowIndex = rowIndex;

			int last;
			for (int selRow : workRows) {

				// checks if row selection is valid
				if (selRow > rows.size())
					throw new IndexOutOfBoundsException(selRow + " > " + "(rows.size() = " + rows.size() + ")");
				if (selRow < 0)
					throw new IndexOutOfBoundsException(selRow + " < 0");

				last = edIntervalIdx.size() - 1;

				// checks if the next selected value conforms an interval
				if (selRow == (edIntervalIdx.get(last) + 1) || selRow == (edIntervalIdx.get(last)))
					edIntervalIdx.set(last, selRow);
				else {
					bgIntervalIdx.add(selRow);
					edIntervalIdx.add(selRow);
				}

				// if a line before the index is selected, when we remove it
				// insertion row index must be decreased by one
				if (rowIndex - 1 >= selRow)
					offsetRowIndex--;
			}

			// join all the rows in the same array and removes them from
			// the table
			List<List<Object>> joinedRows = new ArrayList<List<Object>>();
			List<List<Object>> temp;
			for (int i = bgIntervalIdx.size() - 1; i >= 0; i--) {
				temp = rows.subList(bgIntervalIdx.get(i), edIntervalIdx.get(i) + 1);
				joinedRows.addAll(0, temp);
				temp.clear();
			}

			// add all the object to the specified position
			rows.addAll(offsetRowIndex, joinedRows);

			// adds new moved rows positions to the list
			newPositions = new int[joinedRows.size()];
			for (int i = 0; i < newPositions.length; i++)
				newPositions[i] = offsetRowIndex + i;

		} else {
			newPositions = new int[0];
		}

		fireTableDataChanged();

		return newPositions;

	}

	public int[] moveRowsTo(int rowIndex, int fromRowIndex, int toRowIndex) {

		int[] selRows = new int[toRowIndex - fromRowIndex];
		
		for (int row = fromRowIndex, i = 0; row < toRowIndex; row++, i++)
			selRows[i] = row;
		
		return moveRowsTo(toRowIndex, selRows);
	}

	public int[] moveRowsToTop(int fromRowIndex, int toRowIndex) {

		return moveRowsTo(0, fromRowIndex, toRowIndex);
	}


	public int[] moveRowsToTop(int[] selRows) {

		return moveRowsTo(0, selRows);
	}

	public int[] moveRowsToBottom(int fromRowIndex, int toRowIndex) {

		return moveRowsTo(rows.size() - 1, fromRowIndex, toRowIndex);
	}

	public int[] moveRowsToBottom(int[] selRows) {

		return moveRowsTo(rows.size(), selRows);
	}

	public int[] moveUp(int fromRowIndex, int toRowIndex) {

		int[] newPositions;

		if (fromRowIndex > 0)
			newPositions = moveRowsTo(fromRowIndex - 1, fromRowIndex, toRowIndex);
		else
			newPositions = new int[0];

		return newPositions;
	}

	public int[] moveUp(int[] selRows) {

		Arrays.sort(selRows);
		
		int upestRow = selRows[0];
		
		int[] newPositions;

		if (upestRow > 0)
			newPositions = moveRowsTo(upestRow - 1, selRows);
		else
			newPositions = new int[0];

		return newPositions;
	}

	public int[] moveDown(int fromRowIndex, int toRowIndex) {

		int[] newPositions;

		if (fromRowIndex < rows.size() - 1)
			newPositions = moveRowsTo(fromRowIndex + 2, fromRowIndex, toRowIndex);
		else
			newPositions = new int[0];

		return newPositions;
	}

	public int[] moveDown(int[] selRows) {

		Arrays.sort(selRows);

		int lowerRow = selRows[selRows.length - 1];

		int[] newPositions;

		if (lowerRow < rows.size() - 1)
			newPositions = moveRowsTo(lowerRow + 2, selRows);
		else
			newPositions = new int[0];

		return newPositions;
	}


	private List<Class<?>> getSuperClasses(Class<?> c) {

		List<Class<?>> classes;

		if (!c.equals(Object.class)) {

			classes = getSuperClasses(c.getSuperclass());
			classes.add(c);
		} else {
			classes = new ArrayList<Class<?>>();
			classes.add(c);
		}
		return classes;
	}

	public static void main(String... args) {
		
		ArrayList<String> columns = new ArrayList<String>();
		columns.add("primera");
		
		MoveRowTableModel as = new MoveRowTableModel(columns);
		ArrayList<Object> row1 = new ArrayList<Object>();
		row1.add(new Integer(1));
		as.rows.add(row1);
		ArrayList<Object> row2 = new ArrayList<Object>();
		row2.add(new Integer(1));
		as.rows.add(row2);
		ArrayList<Object> row3 = new ArrayList<Object>();
		row3.add(new Float(1));
		as.rows.add(row3);
		
		System.out.println(as.getColumnClass(0));

	}

}
