//    Copyright (C) 2010  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.gui.auxi;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

//TODO: JavaDoc
public class RowColorCellRenderer extends DefaultTableCellRenderer {
	
	private static final long serialVersionUID = 6918127047526869087L;

	private TableModel tableModel;

	private Map<Integer, Color> rowColors;

	private Integer columnStringContainer;
	private Object changeObject;

	private Color primaryColor;
	private Color secondaryColor;

	public RowColorCellRenderer(TableModel tableModel, Color primaryColor, Color secondaryColor, Integer column, Object changeObject) {
		super();

		this.tableModel = tableModel;
		this.columnStringContainer = column;
		this.changeObject = changeObject;

		this.primaryColor = primaryColor;
		this.secondaryColor = secondaryColor;

		tableModel.addTableModelListener(new UpdateRows(this));
		recolorRows();
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

		if (!table.isCellSelected(row, column))
			c.setBackground(rowColors.get(row));

		return c;
	}
	
	private void recolorRows() {

		rowColors = new HashMap<Integer, Color>();

		boolean paintingPrimary = true;

		for (int i = 0; i < tableModel.getRowCount(); i++) {

			if(tableModel.getValueAt(i, columnStringContainer).equals(changeObject))
				paintingPrimary = !paintingPrimary;

			if (paintingPrimary)
				rowColors.put(i, primaryColor);
			else
				rowColors.put(i, secondaryColor);
		}

	}

	private class UpdateRows implements TableModelListener {

		private RowColorCellRenderer rcr;

		private UpdateRows(RowColorCellRenderer rcr) {
			this.rcr = rcr;
		}

		@Override
		public void tableChanged(TableModelEvent e) {

			rcr.recolorRows();
		}

	}
	
	public static void main(String... s) {
		
		TableModel tm = new DefaultTableModel(new String[][]{{"Uno", "a", "Tres"}, {"Uno", "Dos", "Tres"}, {"Uno", "Dos", "Tres"}, {"Uno", "a", "Tres"}, {"Uno", "a", "Tres"}, {"Uno", "Dos", "Tres"}}, new String[]{"Uno", "Dos", "Tres"});

		final RowColorCellRenderer rcdr = new RowColorCellRenderer(tm, Color.WHITE, Color.LIGHT_GRAY, 1, "AND");

		final JTable jt = new JTable(tm);
		jt.setDefaultRenderer(Object.class, rcdr);

		final JFrame jf = new JFrame();
		jf.setLayout(new GridLayout(4, 1));
		jf.setSize(400, 400);
		jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		jf.add(new JScrollPane(jt));

		JButton repaint = new JButton("Repaint");
		repaint.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				// jt.updateUI();
				jt.repaint();
				// jf.repaint();
			}
		});

		JButton addRenderer = new JButton("AddRenderer");
		addRenderer.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				jt.setDefaultRenderer(Object.class, rcdr);
			}
		});

		JButton enable = new JButton("Enable");
		enable.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				jt.setEnabled(!jt.isEnabled());
			}
		});

		jf.add(repaint);
		jf.add(enable);
		jf.add(addRenderer);
		jf.setVisible(true);
	}
}
